from .const import DATA_COORDINATOR as DATA_COORDINATOR, DATA_TILE as DATA_TILE, DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import aiohttp_client as aiohttp_client
from homeassistant.helpers.entity_registry import RegistryEntry as RegistryEntry, async_migrate_entries as async_migrate_entries
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from homeassistant.util.async_ import gather_with_concurrency as gather_with_concurrency
from pytile.tile import Tile as Tile
from typing import Any

PLATFORMS: Any
DEVICE_TYPES: Any
DEFAULT_INIT_TASK_LIMIT: int
DEFAULT_UPDATE_INTERVAL: Any
CONF_SHOW_INACTIVE: str

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...



from .runner import RuntimeConfig as RuntimeConfig
from homeassistant import config_entries as config_entries, core as core, loader as loader
from homeassistant.components import http as http
from homeassistant.const import REQUIRED_NEXT_PYTHON_DATE as REQUIRED_NEXT_PYTHON_DATE, REQUIRED_NEXT_PYTHON_VER as REQUIRED_NEXT_PYTHON_VER
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers import area_registry as area_registry, device_registry as device_registry, entity_registry as entity_registry
from homeassistant.helpers.dispatcher import async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.setup import DATA_SETUP as DATA_SETUP, DATA_SETUP_STARTED as DATA_SETUP_STARTED, DATA_SETUP_TIME as DATA_SETUP_TIME, async_set_domains_to_be_loaded as async_set_domains_to_be_loaded, async_setup_component as async_setup_component
from homeassistant.util.async_ import gather_with_concurrency as gather_with_concurrency
from homeassistant.util.logging import async_activate_log_queue_handler as async_activate_log_queue_handler
from homeassistant.util.package import async_get_user_site as async_get_user_site, is_virtual_env as is_virtual_env
from typing import Any

_LOGGER: Any
ERROR_LOG_FILENAME: str
DATA_LOGGING: str
LOG_SLOW_STARTUP_INTERVAL: int
SLOW_STARTUP_CHECK_INTERVAL: int
SIGNAL_BOOTSTRAP_INTEGRATONS: str
STAGE_1_TIMEOUT: int
STAGE_2_TIMEOUT: int
WRAP_UP_TIMEOUT: int
COOLDOWN_TIME: int
MAX_LOAD_CONCURRENTLY: int
DEBUGGER_INTEGRATIONS: Any
CORE_INTEGRATIONS: Any
LOGGING_INTEGRATIONS: Any
STAGE_1_INTEGRATIONS: Any

async def async_setup_hass(runtime_config: RuntimeConfig) -> Union[core.HomeAssistant, None]: ...
def open_hass_ui(hass: core.HomeAssistant) -> None: ...
async def async_from_config_dict(config: ConfigType, hass: core.HomeAssistant) -> Union[core.HomeAssistant, None]: ...
def async_enable_logging(hass: core.HomeAssistant, verbose: bool = ..., log_rotate_days: Union[int, None] = ..., log_file: Union[str, None] = ..., log_no_color: bool = ...) -> None: ...
async def async_mount_local_lib_path(config_dir: str) -> str: ...
def _get_domains(hass: core.HomeAssistant, config: dict[str, Any]) -> set[str]: ...
async def _async_watch_pending_setups(hass: core.HomeAssistant) -> None: ...
async def async_setup_multi_components(hass: core.HomeAssistant, domains: set[str], config: dict[str, Any]) -> None: ...
async def _async_set_up_integrations(hass: core.HomeAssistant, config: dict[str, Any]) -> None: ...



from .const import DOMAIN as DOMAIN
from .model import CanaryData as CanaryData
from canary.api import Api as Api, Location as Location
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from typing import Any

_LOGGER: Any

class CanaryDataUpdateCoordinator(DataUpdateCoordinator):
    canary: Any
    def __init__(self, hass: HomeAssistant, api: Api) -> None: ...
    def _update_data(self) -> CanaryData: ...
    async def _async_update_data(self) -> CanaryData: ...



from homeassistant.const import RESTART_EXIT_CODE as RESTART_EXIT_CODE
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.loader import bind_hass as bind_hass
from types import FrameType as FrameType
from typing import Any

_LOGGER: Any

def async_register_signal_handling(hass: HomeAssistant) -> None: ...



from .helpers import on_unload as on_unload
from enum import Enum
from homeassistant.components import mysensors as mysensors
from homeassistant.components.cover import ATTR_POSITION as ATTR_POSITION, CoverEntity as CoverEntity, DOMAIN as DOMAIN
from homeassistant.components.mysensors.const import DiscoveryInfo as DiscoveryInfo, MYSENSORS_DISCOVERY as MYSENSORS_DISCOVERY
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import STATE_OFF as STATE_OFF, STATE_ON as STATE_ON
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

_LOGGER: Any

class CoverState(Enum):
    OPEN: int
    OPENING: int
    CLOSING: int
    CLOSED: int

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class MySensorsCover(mysensors.device.MySensorsEntity, CoverEntity):
    def get_cover_state(self) -> CoverState: ...
    @property
    def is_closed(self) -> bool: ...
    @property
    def is_closing(self) -> bool: ...
    @property
    def is_opening(self) -> bool: ...
    @property
    def current_cover_position(self) -> Union[int, None]: ...
    async def async_open_cover(self, **kwargs: Any) -> None: ...
    async def async_close_cover(self, **kwargs: Any) -> None: ...
    async def async_set_cover_position(self, **kwargs: Any) -> None: ...
    async def async_stop_cover(self, **kwargs: Any) -> None: ...



from typing import Final

DOMAIN: Final[str]
MANUFACTURER: Final[str]
CONF_FFMPEG_ARGUMENTS: Final[str]
DATA_COORDINATOR: Final[str]
DATA_UNDO_UPDATE_LISTENER: Final[str]
DEFAULT_FFMPEG_ARGUMENTS: Final[str]
DEFAULT_TIMEOUT: Final[int]



from .const import CHILD_CALLBACK as CHILD_CALLBACK, DevId as DevId, GatewayId as GatewayId, NODE_CALLBACK as NODE_CALLBACK
from .device import get_mysensors_devices as get_mysensors_devices
from .helpers import discover_mysensors_platform as discover_mysensors_platform, validate_set_msg as validate_set_msg
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_send as async_dispatcher_send
from homeassistant.util import decorator as decorator
from mysensors import Message as Message
from typing import Any

HANDLERS: Any

async def handle_set(hass: HomeAssistant, gateway_id: GatewayId, msg: Message) -> None: ...
async def handle_internal(hass: HomeAssistant, gateway_id: GatewayId, msg: Message) -> None: ...
async def handle_battery_level(hass: HomeAssistant, gateway_id: GatewayId, msg: Message) -> None: ...
async def handle_heartbeat(hass: HomeAssistant, gateway_id: GatewayId, msg: Message) -> None: ...
async def handle_sketch_name(hass: HomeAssistant, gateway_id: GatewayId, msg: Message) -> None: ...
async def handle_sketch_version(hass: HomeAssistant, gateway_id: GatewayId, msg: Message) -> None: ...
def _handle_child_update(hass: HomeAssistant, gateway_id: GatewayId, validated: dict[str, list[DevId]]) -> None: ...
def _handle_node_update(hass: HomeAssistant, gateway_id: GatewayId, msg: Message) -> None: ...



from typing import Any

COLORS: Any

class XYPoint:
    x: float
    y: float
    def __init__(self, x, y) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

class GamutType:
    red: XYPoint
    green: XYPoint
    blue: XYPoint
    def __init__(self, red, green, blue) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

def color_name_to_rgb(color_name: str) -> tuple[int, int, int]: ...
def color_RGB_to_xy(iR: int, iG: int, iB: int, Gamut: Union[GamutType, None] = ...) -> tuple[float, float]: ...
def color_RGB_to_xy_brightness(iR: int, iG: int, iB: int, Gamut: Union[GamutType, None] = ...) -> tuple[float, float, int]: ...
def color_xy_to_RGB(vX: float, vY: float, Gamut: Union[GamutType, None] = ...) -> tuple[int, int, int]: ...
def color_xy_brightness_to_RGB(vX: float, vY: float, ibrightness: int, Gamut: Union[GamutType, None] = ...) -> tuple[int, int, int]: ...
def color_hsb_to_RGB(fH: float, fS: float, fB: float) -> tuple[int, int, int]: ...
def color_RGB_to_hsv(iR: float, iG: float, iB: float) -> tuple[float, float, float]: ...
def color_RGB_to_hs(iR: float, iG: float, iB: float) -> tuple[float, float]: ...
def color_hsv_to_RGB(iH: float, iS: float, iV: float) -> tuple[int, int, int]: ...
def color_hs_to_RGB(iH: float, iS: float) -> tuple[int, int, int]: ...
def color_xy_to_hs(vX: float, vY: float, Gamut: Union[GamutType, None] = ...) -> tuple[float, float]: ...
def color_hs_to_xy(iH: float, iS: float, Gamut: Union[GamutType, None] = ...) -> tuple[float, float]: ...
def _match_max_scale(input_colors: tuple, output_colors: tuple) -> tuple: ...
def color_rgb_to_rgbw(r: int, g: int, b: int) -> tuple[int, int, int, int]: ...
def color_rgbw_to_rgb(r: int, g: int, b: int, w: int) -> tuple[int, int, int]: ...
def color_rgb_to_rgbww(r: int, g: int, b: int, min_mireds: int, max_mireds: int) -> tuple[int, int, int, int, int]: ...
def color_rgbww_to_rgb(r: int, g: int, b: int, cw: int, ww: int, min_mireds: int, max_mireds: int) -> tuple[int, int, int]: ...
def color_rgb_to_hex(r: int, g: int, b: int) -> str: ...
def rgb_hex_to_rgb_list(hex_string: str) -> list[int]: ...
def color_temperature_to_hs(color_temperature_kelvin: float) -> tuple[float, float]: ...
def color_temperature_to_rgb(color_temperature_kelvin: float) -> tuple[float, float, float]: ...
def _clamp(color_component: float, minimum: float = ..., maximum: float = ...) -> float: ...
def _get_red(temperature: float) -> float: ...
def _get_green(temperature: float) -> float: ...
def _get_blue(temperature: float) -> float: ...
def color_temperature_mired_to_kelvin(mired_temperature: float) -> int: ...
def color_temperature_kelvin_to_mired(kelvin_temperature: float) -> int: ...
def cross_product(p1: XYPoint, p2: XYPoint) -> float: ...
def get_distance_between_two_points(one: XYPoint, two: XYPoint) -> float: ...
def get_closest_point_to_line(A: XYPoint, B: XYPoint, P: XYPoint) -> XYPoint: ...
def get_closest_point_to_point(xy_tuple: tuple[float, float], Gamut: GamutType) -> tuple[float, float]: ...
def check_point_in_lamps_reach(p: tuple[float, float], Gamut: GamutType) -> bool: ...
def check_valid_gamut(Gamut: GamutType) -> bool: ...



from typing import Final

SUPPORT_ALARM_ARM_HOME: Final[int]
SUPPORT_ALARM_ARM_AWAY: Final[int]
SUPPORT_ALARM_ARM_NIGHT: Final[int]
SUPPORT_ALARM_TRIGGER: Final[int]
SUPPORT_ALARM_ARM_CUSTOM_BYPASS: Final[int]
SUPPORT_ALARM_ARM_VACATION: Final[int]
CONDITION_TRIGGERED: Final[str]
CONDITION_DISARMED: Final[str]
CONDITION_ARMED_HOME: Final[str]
CONDITION_ARMED_AWAY: Final[str]
CONDITION_ARMED_NIGHT: Final[str]
CONDITION_ARMED_VACATION: Final[str]
CONDITION_ARMED_CUSTOM_BYPASS: Final[str]



from .const import DATA_BRIDGE as DATA_BRIDGE, DISCOVERY_TIME_SEC as DISCOVERY_TIME_SEC, DOMAIN as DOMAIN
from aioswitcher.bridge import SwitcherBase as SwitcherBase
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from typing import Any, Callable

_LOGGER: Any

async def async_start_bridge(hass: HomeAssistant, on_device_callback: Callable[[SwitcherBase], Any]) -> None: ...
async def async_stop_bridge(hass: HomeAssistant) -> None: ...
async def async_discover_devices() -> dict[str, SwitcherBase]: ...



from .const import DOMAIN as DOMAIN, PLATFORMS as PLATFORMS
from .onewirehub import CannotConnect as CannotConnect, OneWireHub as OneWireHub
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from typing import Any

_LOGGER: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...



from typing import Any

CONF_AUTH_ID: str
CONF_CREATE_TOKEN: str
CONF_INSTANCE: str
CONF_INSTANCE_CLIENTS: str
CONF_ON_UNLOAD: str
CONF_PRIORITY: str
CONF_ROOT_CLIENT: str
CONF_EFFECT_HIDE_LIST: str
CONF_EFFECT_SHOW_LIST: str
DEFAULT_NAME: str
DEFAULT_ORIGIN: str
DEFAULT_PRIORITY: int
DOMAIN: str
HYPERION_MANUFACTURER_NAME: str
HYPERION_MODEL_NAME: Any
HYPERION_RELEASES_URL: str
HYPERION_VERSION_WARN_CUTOFF: str
NAME_SUFFIX_HYPERION_LIGHT: str
NAME_SUFFIX_HYPERION_PRIORITY_LIGHT: str
NAME_SUFFIX_HYPERION_COMPONENT_SWITCH: str
NAME_SUFFIX_HYPERION_CAMERA: str
SIGNAL_INSTANCE_ADD: Any
SIGNAL_INSTANCE_REMOVE: Any
SIGNAL_ENTITY_REMOVE: Any
TYPE_HYPERION_CAMERA: str
TYPE_HYPERION_LIGHT: str
TYPE_HYPERION_PRIORITY_LIGHT: str
TYPE_HYPERION_COMPONENT_SWITCH_BASE: str



import aioshelly
from . import ShellyDeviceRestWrapper as ShellyDeviceRestWrapper, ShellyDeviceWrapper as ShellyDeviceWrapper
from .const import AIOSHELLY_DEVICE_TIMEOUT_SEC as AIOSHELLY_DEVICE_TIMEOUT_SEC, COAP as COAP, DATA_CONFIG_ENTRY as DATA_CONFIG_ENTRY, DOMAIN as DOMAIN, REST as REST
from .utils import async_remove_shelly_entity as async_remove_shelly_entity, get_entity_name as get_entity_name
from homeassistant.components.sensor import ATTR_STATE_CLASS as ATTR_STATE_CLASS
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import device_registry as device_registry, entity as entity, entity_registry as entity_registry, update_coordinator as update_coordinator
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.restore_state import RestoreEntity as RestoreEntity
from homeassistant.helpers.typing import StateType as StateType
from typing import Any, Callable, Final

_LOGGER: Final[Any]

async def async_setup_entry_attribute_entities(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback, sensors: dict[tuple[str, str], BlockAttributeDescription], sensor_class: Callable) -> None: ...
async def async_setup_block_attribute_entities(hass: HomeAssistant, async_add_entities: AddEntitiesCallback, wrapper: ShellyDeviceWrapper, sensors: dict[tuple[str, str], BlockAttributeDescription], sensor_class: Callable) -> None: ...
async def async_restore_block_attribute_entities(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback, wrapper: ShellyDeviceWrapper, sensors: dict[tuple[str, str], BlockAttributeDescription], sensor_class: Callable) -> None: ...
async def async_setup_entry_rest(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback, sensors: dict[str, RestAttributeDescription], sensor_class: Callable) -> None: ...

class BlockAttributeDescription:
    name: str
    icon: Union[str, None]
    unit: Union[None, str, Callable[[dict], str]]
    value: Callable[[Any], Any]
    device_class: Union[str, None]
    state_class: Union[str, None]
    default_enabled: bool
    available: Union[Callable[[aioshelly.Block], bool], None]
    removal_condition: Union[Callable[[dict, aioshelly.Block], bool], None]
    extra_state_attributes: Union[Callable[[aioshelly.Block], Union[dict, None]], None]
    last_reset: Union[str, None]

class RestAttributeDescription:
    name: str
    icon: Union[str, None]
    unit: Union[str, None]
    value: Union[Callable[[dict, Any], Any], None]
    device_class: Union[str, None]
    state_class: Union[str, None]
    default_enabled: bool
    extra_state_attributes: Union[Callable[[dict], Union[dict, None]], None]

class ShellyBlockEntity(entity.Entity):
    wrapper: Any
    block: Any
    _name: Any
    def __init__(self, wrapper: ShellyDeviceWrapper, block: aioshelly.Block) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def should_poll(self) -> bool: ...
    @property
    def device_info(self) -> DeviceInfo: ...
    @property
    def available(self) -> bool: ...
    @property
    def unique_id(self) -> str: ...
    async def async_added_to_hass(self) -> None: ...
    async def async_update(self) -> None: ...
    def _update_callback(self) -> None: ...
    async def set_state(self, **kwargs: Any) -> Any: ...

class ShellyBlockAttributeEntity(ShellyBlockEntity, entity.Entity):
    attribute: Any
    description: Any
    _unit: Any
    _unique_id: Any
    _name: Any
    def __init__(self, wrapper: ShellyDeviceWrapper, block: aioshelly.Block, attribute: str, description: BlockAttributeDescription) -> None: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def entity_registry_enabled_default(self) -> bool: ...
    @property
    def attribute_value(self) -> StateType: ...
    @property
    def device_class(self) -> Union[str, None]: ...
    @property
    def icon(self) -> Union[str, None]: ...
    @property
    def available(self) -> bool: ...
    @property
    def extra_state_attributes(self) -> Union[dict[str, Any], None]: ...

class ShellyRestAttributeEntity(update_coordinator.CoordinatorEntity):
    wrapper: Any
    attribute: Any
    description: Any
    _name: Any
    _last_value: Any
    def __init__(self, wrapper: ShellyDeviceWrapper, attribute: str, description: RestAttributeDescription) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def device_info(self) -> DeviceInfo: ...
    @property
    def entity_registry_enabled_default(self) -> bool: ...
    @property
    def available(self) -> bool: ...
    @property
    def attribute_value(self) -> StateType: ...
    @property
    def device_class(self) -> Union[str, None]: ...
    @property
    def icon(self) -> Union[str, None]: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def extra_state_attributes(self) -> Union[dict[str, Any], None]: ...

class ShellySleepingBlockAttributeEntity(ShellyBlockAttributeEntity, RestoreEntity):
    sensors: Any
    last_state: Any
    wrapper: Any
    attribute: Any
    block: Any
    description: Any
    _unit: Any
    _unique_id: Any
    _name: Any
    def __init__(self, wrapper: ShellyDeviceWrapper, block: aioshelly.Block, attribute: str, description: BlockAttributeDescription, entry: Union[entity_registry.RegistryEntry, None] = ..., sensors: Union[dict[tuple[str, str], BlockAttributeDescription], None] = ...) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    def _update_callback(self) -> None: ...



from .const import DOMAIN as DOMAIN
from .devolo_device import DevoloDeviceEntity as DevoloDeviceEntity
from devolo_home_control_api.devices.zwave import Zwave as Zwave
from devolo_home_control_api.homecontrol import HomeControl as HomeControl
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity, DEVICE_CLASS_DOOR as DEVICE_CLASS_DOOR, DEVICE_CLASS_HEAT as DEVICE_CLASS_HEAT, DEVICE_CLASS_MOISTURE as DEVICE_CLASS_MOISTURE, DEVICE_CLASS_MOTION as DEVICE_CLASS_MOTION, DEVICE_CLASS_PROBLEM as DEVICE_CLASS_PROBLEM, DEVICE_CLASS_SAFETY as DEVICE_CLASS_SAFETY, DEVICE_CLASS_SMOKE as DEVICE_CLASS_SMOKE
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

DEVICE_CLASS_MAPPING: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class DevoloBinaryDeviceEntity(DevoloDeviceEntity, BinarySensorEntity):
    _binary_sensor_property: Any
    _attr_device_class: Any
    _value: Any
    _attr_entity_registry_enabled_default: bool
    def __init__(self, homecontrol: HomeControl, device_instance: Zwave, element_uid: str) -> None: ...
    @property
    def is_on(self) -> bool: ...

class DevoloRemoteControl(DevoloDeviceEntity, BinarySensorEntity):
    _remote_control_property: Any
    _key: Any
    _attr_is_on: bool
    def __init__(self, homecontrol: HomeControl, device_instance: Zwave, element_uid: str, key: int) -> None: ...
    def _sync(self, message: tuple) -> None: ...



from . import template as template
from collections.abc import Mapping
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from typing import Any

class ScriptVariables:
    variables: Any
    _has_template: Any
    def __init__(self, variables: dict[str, Any]) -> None: ...
    def async_render(self, hass: HomeAssistant, run_variables: Union[Mapping[str, Any], None], *, render_as_defaults: bool = ..., limited: bool = ...) -> dict[str, Any]: ...
    def as_dict(self) -> dict: ...



from . import SynoApi as SynoApi, SynologyDSMBaseEntity as SynologyDSMBaseEntity, SynologyDSMDeviceEntity as SynologyDSMDeviceEntity
from .const import CONF_VOLUMES as CONF_VOLUMES, COORDINATOR_CENTRAL as COORDINATOR_CENTRAL, DOMAIN as DOMAIN, ENTITY_UNIT_LOAD as ENTITY_UNIT_LOAD, EntityInfo as EntityInfo, INFORMATION_SENSORS as INFORMATION_SENSORS, STORAGE_DISK_SENSORS as STORAGE_DISK_SENSORS, STORAGE_VOL_SENSORS as STORAGE_VOL_SENSORS, SYNO_API as SYNO_API, TEMP_SENSORS_KEYS as TEMP_SENSORS_KEYS, UTILISATION_SENSORS as UTILISATION_SENSORS
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_DISKS as CONF_DISKS, DATA_MEGABYTES as DATA_MEGABYTES, DATA_RATE_KILOBYTES_PER_SECOND as DATA_RATE_KILOBYTES_PER_SECOND, DATA_TERABYTES as DATA_TERABYTES, PRECISION_TENTHS as PRECISION_TENTHS, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.temperature import display_temp as display_temp
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from homeassistant.util.dt import utcnow as utcnow
from typing import Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class SynoDSMSensor(SynologyDSMBaseEntity):
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...

class SynoDSMUtilSensor(SynoDSMSensor, SensorEntity):
    @property
    def state(self) -> Union[Any, None]: ...
    @property
    def available(self) -> bool: ...

class SynoDSMStorageSensor(SynologyDSMDeviceEntity, SynoDSMSensor, SensorEntity):
    @property
    def state(self) -> Union[Any, None]: ...

class SynoDSMInfoSensor(SynoDSMSensor, SensorEntity):
    _previous_uptime: Any
    _last_boot: Any
    def __init__(self, api: SynoApi, entity_type: str, entity_info: EntityInfo, coordinator: DataUpdateCoordinator[dict[str, dict[str, Any]]]) -> None: ...
    @property
    def state(self) -> Union[Any, None]: ...



from .const import DOMAIN as DOMAIN
from .data import EnergyManager as EnergyManager, async_get_manager as async_get_manager
from homeassistant.components.sensor import ATTR_LAST_RESET as ATTR_LAST_RESET, DEVICE_CLASS_MONETARY as DEVICE_CLASS_MONETARY, STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT, SensorEntity as SensorEntity
from homeassistant.const import ATTR_UNIT_OF_MEASUREMENT as ATTR_UNIT_OF_MEASUREMENT, ENERGY_KILO_WATT_HOUR as ENERGY_KILO_WATT_HOUR, ENERGY_WATT_HOUR as ENERGY_WATT_HOUR
from homeassistant.core import HomeAssistant as HomeAssistant, State as State, callback as callback, split_entity_id as split_entity_id
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any, Final, Literal, TypeVar

_LOGGER: Any

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...
T = TypeVar('T')

class FlowAdapter:
    flow_type: Literal[flow_from, flow_to]
    stat_energy_key: Literal[stat_energy_from, stat_energy_to]
    entity_energy_key: Literal[entity_energy_from, entity_energy_to]
    total_money_key: Literal[stat_cost, stat_compensation]
    name_suffix: str
    entity_id_suffix: str

FLOW_ADAPTERS: Final[Any]

async def _process_manager_data(hass: HomeAssistant, manager: EnergyManager, async_add_entities: AddEntitiesCallback, current_entities: dict[tuple[str, str], EnergyCostSensor]) -> None: ...

class EnergyCostSensor(SensorEntity):
    _adapter: Any
    entity_id: Any
    _attr_device_class: Any
    _attr_state_class: Any
    _flow: Any
    _last_energy_sensor_state: Any
    _cur_value: float
    def __init__(self, adapter: FlowAdapter, flow: dict) -> None: ...
    _attr_state: float
    _attr_last_reset: Any
    def _reset(self, energy_state: State) -> None: ...
    def _update_cost(self) -> None: ...
    _attr_name: Any
    async def async_added_to_hass(self) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    def update_config(self, flow: dict) -> None: ...
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...



import datetime as dt
import logging
from .auth import AuthPhase as AuthPhase, auth_required_message as auth_required_message
from .const import CANCELLATION_ERRORS as CANCELLATION_ERRORS, DATA_CONNECTIONS as DATA_CONNECTIONS, MAX_PENDING_MSG as MAX_PENDING_MSG, PENDING_MSG_PEAK as PENDING_MSG_PEAK, PENDING_MSG_PEAK_TIME as PENDING_MSG_PEAK_TIME, SIGNAL_WEBSOCKET_CONNECTED as SIGNAL_WEBSOCKET_CONNECTED, SIGNAL_WEBSOCKET_DISCONNECTED as SIGNAL_WEBSOCKET_DISCONNECTED, URL as URL
from .error import Disconnect as Disconnect
from .messages import message_to_json as message_to_json
from aiohttp import web
from collections.abc import Callable as Callable
from homeassistant.components.http import HomeAssistantView as HomeAssistantView
from homeassistant.const import EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.event import async_call_later as async_call_later
from typing import Any, Final

_WS_LOGGER: Final[Any]

class WebsocketAPIView(HomeAssistantView):
    name: str
    url: str
    requires_auth: bool
    async def get(self, request: web.Request) -> web.WebSocketResponse: ...

class WebSocketAdapter(logging.LoggerAdapter):
    def process(self, msg: str, kwargs: Any) -> tuple[str, Any]: ...

class WebSocketHandler:
    hass: Any
    request: Any
    wsock: Any
    _to_write: Any
    _handle_task: Any
    _writer_task: Any
    _logger: Any
    _peak_checker_unsub: Any
    def __init__(self, hass: HomeAssistant, request: web.Request) -> None: ...
    async def _writer(self) -> None: ...
    def _send_message(self, message: Union[str, dict[str, Any]]) -> None: ...
    def _check_write_peak(self, _utc_time: dt.datetime) -> None: ...
    def _cancel(self) -> None: ...
    async def async_handle(self) -> web.WebSocketResponse: ...



import abc
import logging
from .aiohttp_client import async_get_clientsession as async_get_clientsession
from abc import ABC, ABCMeta, abstractmethod
from aiohttp import client as client, web
from collections.abc import Awaitable
from homeassistant import config_entries as config_entries
from homeassistant.components import http as http
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.network import NoURLAvailableError as NoURLAvailableError
from typing import Any, Callable

_LOGGER: Any
DATA_JWT_SECRET: str
DATA_VIEW_REGISTERED: str
DATA_IMPLEMENTATIONS: str
DATA_PROVIDERS: str
AUTH_CALLBACK_PATH: str
HEADER_FRONTEND_BASE: str
CLOCK_OUT_OF_SYNC_MAX_SEC: int

class AbstractOAuth2Implementation(ABC, metaclass=abc.ABCMeta):
    @property
    @abstractmethod
    def name(self) -> str: ...
    @property
    @abstractmethod
    def domain(self) -> str: ...
    @abstractmethod
    async def async_generate_authorize_url(self, flow_id: str) -> str: ...
    @abstractmethod
    async def async_resolve_external_data(self, external_data: Any) -> dict: ...
    async def async_refresh_token(self, token: dict) -> dict: ...
    @abstractmethod
    async def _async_refresh_token(self, token: dict) -> dict: ...

class LocalOAuth2Implementation(AbstractOAuth2Implementation):
    hass: Any
    _domain: Any
    client_id: Any
    client_secret: Any
    authorize_url: Any
    token_url: Any
    def __init__(self, hass: HomeAssistant, domain: str, client_id: str, client_secret: str, authorize_url: str, token_url: str) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def domain(self) -> str: ...
    @property
    def redirect_uri(self) -> str: ...
    @property
    def extra_authorize_data(self) -> dict: ...
    async def async_generate_authorize_url(self, flow_id: str) -> str: ...
    async def async_resolve_external_data(self, external_data: Any) -> dict: ...
    async def _async_refresh_token(self, token: dict) -> dict: ...
    async def _token_request(self, data: dict) -> dict: ...

class AbstractOAuth2FlowHandler(config_entries.ConfigFlow, metaclass=ABCMeta):
    DOMAIN: str
    VERSION: int
    external_data: Any
    flow_impl: Any
    def __init__(self) -> None: ...
    @property
    @abstractmethod
    def logger(self) -> logging.Logger: ...
    @property
    def extra_authorize_data(self) -> dict: ...
    async def async_step_pick_implementation(self, user_input: Union[dict, None] = ...) -> FlowResult: ...
    async def async_step_auth(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_creation(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_oauth_create_entry(self, data: dict) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    @classmethod
    def async_register_implementation(cls, hass: HomeAssistant, local_impl: LocalOAuth2Implementation) -> None: ...

def async_register_implementation(hass: HomeAssistant, domain: str, implementation: AbstractOAuth2Implementation) -> None: ...
async def async_get_implementations(hass: HomeAssistant, domain: str) -> dict[str, AbstractOAuth2Implementation]: ...
async def async_get_config_entry_implementation(hass: HomeAssistant, config_entry: config_entries.ConfigEntry) -> AbstractOAuth2Implementation: ...
def async_add_implementation_provider(hass: HomeAssistant, provider_domain: str, async_provide_implementation: Callable[[HomeAssistant, str], Awaitable[Union[AbstractOAuth2Implementation, None]]]) -> None: ...

class OAuth2AuthorizeCallbackView(http.HomeAssistantView):
    requires_auth: bool
    url: Any
    name: str
    async def get(self, request: web.Request) -> web.Response: ...

class OAuth2Session:
    hass: Any
    config_entry: Any
    implementation: Any
    def __init__(self, hass: HomeAssistant, config_entry: config_entries.ConfigEntry, implementation: AbstractOAuth2Implementation) -> None: ...
    @property
    def token(self) -> dict: ...
    @property
    def valid_token(self) -> bool: ...
    async def async_ensure_token_valid(self) -> None: ...
    async def async_request(self, method: str, url: str, **kwargs: Any) -> client.ClientResponse: ...

async def async_oauth2_request(hass: HomeAssistant, token: dict, method: str, url: str, **kwargs: Any) -> client.ClientResponse: ...
def _encode_jwt(hass: HomeAssistant, data: dict) -> str: ...
def _decode_jwt(hass: HomeAssistant, encoded: str) -> Union[dict, None]: ...



from .const import ATTR_HOST_NAME as ATTR_HOST_NAME, ATTR_IP as ATTR_IP, ATTR_MAC as ATTR_MAC, ATTR_SOURCE_TYPE as ATTR_SOURCE_TYPE, DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant.components import zone as zone
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_BATTERY_LEVEL as ATTR_BATTERY_LEVEL, ATTR_GPS_ACCURACY as ATTR_GPS_ACCURACY, ATTR_LATITUDE as ATTR_LATITUDE, ATTR_LONGITUDE as ATTR_LONGITUDE, STATE_HOME as STATE_HOME, STATE_NOT_HOME as STATE_NOT_HOME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.typing import StateType as StateType

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class BaseTrackerEntity(Entity):
    @property
    def battery_level(self) -> Union[int, None]: ...
    @property
    def source_type(self) -> str: ...
    @property
    def state_attributes(self) -> dict[str, StateType]: ...

class TrackerEntity(BaseTrackerEntity):
    @property
    def should_poll(self) -> bool: ...
    @property
    def force_update(self) -> bool: ...
    @property
    def location_accuracy(self) -> int: ...
    @property
    def location_name(self) -> Union[str, None]: ...
    @property
    def latitude(self) -> Union[float, None]: ...
    @property
    def longitude(self) -> Union[float, None]: ...
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def state_attributes(self) -> dict[str, StateType]: ...

class ScannerEntity(BaseTrackerEntity):
    @property
    def ip_address(self) -> Union[str, None]: ...
    @property
    def mac_address(self) -> Union[str, None]: ...
    @property
    def hostname(self) -> Union[str, None]: ...
    @property
    def state(self) -> str: ...
    @property
    def is_connected(self) -> bool: ...
    @property
    def state_attributes(self) -> dict[str, StateType]: ...



from typing import Any

DOMAIN: str
PLATFORMS: Any
DATA_FRITZ: str
DEFAULT_DEVICE_NAME: str
DEFAULT_HOST: str
DEFAULT_PORT: int
DEFAULT_USERNAME: str
ERROR_AUTH_INVALID: str
ERROR_CANNOT_CONNECT: str
ERROR_UNKNOWN: str
FRITZ_SERVICES: str
SERVICE_REBOOT: str
SERVICE_RECONNECT: str
SWITCH_TYPE_DEFLECTION: str
SWITCH_TYPE_PORTFORWARD: str
SWITCH_TYPE_WIFINETWORK: str
TRACKER_SCAN_INTERVAL: int
UPTIME_DEVIATION: int



from .const import DOMAIN as DOMAIN
from homeassistant.config_entries import ConfigEntry as ConfigEntry, ConfigFlow as ConfigFlow
from homeassistant.const import CONF_API_KEY as CONF_API_KEY, CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE, CONF_NAME as CONF_NAME
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from typing import Any

class AmbeeFlowHandler(ConfigFlow):
    VERSION: int
    entry: Union[ConfigEntry, None]
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_reauth(self, data: dict[str, Any]) -> FlowResult: ...
    async def async_step_reauth_confirm(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



import types
import voluptuous as vol
from homeassistant import data_entry_flow as data_entry_flow, requirements as requirements
from homeassistant.const import CONF_ID as CONF_ID, CONF_NAME as CONF_NAME, CONF_TYPE as CONF_TYPE
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.util.decorator import Registry as Registry
from typing import Any

MULTI_FACTOR_AUTH_MODULES: Any
MULTI_FACTOR_AUTH_MODULE_SCHEMA: Any
DATA_REQS: str
_LOGGER: Any

class MultiFactorAuthModule:
    DEFAULT_TITLE: str
    MAX_RETRY_TIME: int
    hass: Any
    config: Any
    def __init__(self, hass: HomeAssistant, config: dict[str, Any]) -> None: ...
    @property
    def id(self) -> str: ...
    @property
    def type(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def input_schema(self) -> vol.Schema: ...
    async def async_setup_flow(self, user_id: str) -> SetupFlow: ...
    async def async_setup_user(self, user_id: str, setup_data: Any) -> Any: ...
    async def async_depose_user(self, user_id: str) -> None: ...
    async def async_is_user_setup(self, user_id: str) -> bool: ...
    async def async_validate(self, user_id: str, user_input: dict[str, Any]) -> bool: ...

class SetupFlow(data_entry_flow.FlowHandler):
    _auth_module: Any
    _setup_schema: Any
    _user_id: Any
    def __init__(self, auth_module: MultiFactorAuthModule, setup_schema: vol.Schema, user_id: str) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...

async def auth_mfa_module_from_config(hass: HomeAssistant, config: dict[str, Any]) -> MultiFactorAuthModule: ...
async def _load_mfa_module(hass: HomeAssistant, module_name: str) -> types.ModuleType: ...



from .const import AUTH as AUTH, DOMAIN as DOMAIN, MANUFACTURER as MANUFACTURER, WEBHOOK_ACTIVATION as WEBHOOK_ACTIVATION, WEBHOOK_DEACTIVATION as WEBHOOK_DEACTIVATION, WEBHOOK_NACAMERA_CONNECTION as WEBHOOK_NACAMERA_CONNECTION, WEBHOOK_PUSH_TYPE as WEBHOOK_PUSH_TYPE
from datetime import timedelta
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.event import async_track_time_interval as async_track_time_interval
from typing import Any

_LOGGER: Any
CAMERA_DATA_CLASS_NAME: str
WEATHERSTATION_DATA_CLASS_NAME: str
HOMECOACH_DATA_CLASS_NAME: str
HOMEDATA_DATA_CLASS_NAME: str
HOMESTATUS_DATA_CLASS_NAME: str
PUBLICDATA_DATA_CLASS_NAME: str
DATA_CLASSES: Any
BATCH_SIZE: int
DEFAULT_INTERVALS: Any
SCAN_INTERVAL: int

class NetatmoDataClass:
    name: str
    interval: int
    next_scan: float
    subscriptions: list[CALLBACK_TYPE]

class NetatmoDataHandler:
    hass: Any
    _auth: Any
    listeners: Any
    data_classes: Any
    data: Any
    _queue: Any
    _webhook: bool
    def __init__(self, hass: HomeAssistant, entry: ConfigEntry) -> None: ...
    async def async_setup(self) -> None: ...
    async def async_update(self, event_time: timedelta) -> None: ...
    def async_force_update(self, data_class_entry: str) -> None: ...
    async def async_cleanup(self) -> None: ...
    async def handle_event(self, event: dict) -> None: ...
    async def async_fetch_data(self, data_class_entry: str) -> None: ...
    async def register_data_class(self, data_class_name: str, data_class_entry: str, update_callback: CALLBACK_TYPE, **kwargs: Any) -> None: ...
    async def unregister_data_class(self, data_class_entry: str, update_callback: Union[CALLBACK_TYPE, None]) -> None: ...
    @property
    def webhook(self) -> bool: ...



from .const import DEVICE_CLASS_CHARGE_MODE as DEVICE_CLASS_CHARGE_MODE, DEVICE_CLASS_CHARGE_STATE as DEVICE_CLASS_CHARGE_STATE, DEVICE_CLASS_PLUG_STATE as DEVICE_CLASS_PLUG_STATE, DOMAIN as DOMAIN
from .renault_entities import RenaultBatteryDataEntity as RenaultBatteryDataEntity, RenaultChargeModeDataEntity as RenaultChargeModeDataEntity, RenaultCockpitDataEntity as RenaultCockpitDataEntity, RenaultDataEntity as RenaultDataEntity, RenaultHVACDataEntity as RenaultHVACDataEntity
from .renault_hub import RenaultHub as RenaultHub
from .renault_vehicle import RenaultVehicleProxy as RenaultVehicleProxy
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import DEVICE_CLASS_BATTERY as DEVICE_CLASS_BATTERY, DEVICE_CLASS_ENERGY as DEVICE_CLASS_ENERGY, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, LENGTH_KILOMETERS as LENGTH_KILOMETERS, PERCENTAGE as PERCENTAGE, POWER_KILO_WATT as POWER_KILO_WATT, TEMP_CELSIUS as TEMP_CELSIUS, TIME_MINUTES as TIME_MINUTES, VOLUME_LITERS as VOLUME_LITERS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.icon import icon_for_battery_level as icon_for_battery_level
from homeassistant.util import slugify as slugify
from typing import Any

ATTR_BATTERY_AVAILABLE_ENERGY: str

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...
async def get_entities(proxy: RenaultHub) -> list[RenaultDataEntity]: ...
async def get_vehicle_entities(vehicle: RenaultVehicleProxy) -> list[RenaultDataEntity]: ...

class RenaultBatteryAutonomySensor(RenaultBatteryDataEntity, SensorEntity):
    _attr_icon: str
    _attr_unit_of_measurement: Any
    @property
    def state(self) -> Union[int, None]: ...

class RenaultBatteryLevelSensor(RenaultBatteryDataEntity, SensorEntity):
    _attr_device_class: Any
    _attr_unit_of_measurement: Any
    @property
    def state(self) -> Union[int, None]: ...
    @property
    def icon(self) -> str: ...
    @property
    def extra_state_attributes(self) -> dict[str, Any]: ...

class RenaultBatteryTemperatureSensor(RenaultBatteryDataEntity, SensorEntity):
    _attr_device_class: Any
    _attr_unit_of_measurement: Any
    @property
    def state(self) -> Union[int, None]: ...

class RenaultChargeModeSensor(RenaultChargeModeDataEntity, SensorEntity):
    _attr_device_class: Any
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def icon(self) -> str: ...

class RenaultChargeStateSensor(RenaultBatteryDataEntity, SensorEntity):
    _attr_device_class: Any
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def icon(self) -> str: ...

class RenaultChargingRemainingTimeSensor(RenaultBatteryDataEntity, SensorEntity):
    _attr_icon: str
    _attr_unit_of_measurement: Any
    @property
    def state(self) -> Union[int, None]: ...

class RenaultChargingPowerSensor(RenaultBatteryDataEntity, SensorEntity):
    _attr_device_class: Any
    _attr_unit_of_measurement: Any
    @property
    def state(self) -> Union[float, None]: ...

class RenaultFuelAutonomySensor(RenaultCockpitDataEntity, SensorEntity):
    _attr_icon: str
    _attr_unit_of_measurement: Any
    @property
    def state(self) -> Union[int, None]: ...

class RenaultFuelQuantitySensor(RenaultCockpitDataEntity, SensorEntity):
    _attr_icon: str
    _attr_unit_of_measurement: Any
    @property
    def state(self) -> Union[int, None]: ...

class RenaultMileageSensor(RenaultCockpitDataEntity, SensorEntity):
    _attr_icon: str
    _attr_unit_of_measurement: Any
    @property
    def state(self) -> Union[int, None]: ...

class RenaultOutsideTemperatureSensor(RenaultHVACDataEntity, SensorEntity):
    _attr_device_class: Any
    _attr_unit_of_measurement: Any
    @property
    def state(self) -> Union[float, None]: ...

class RenaultPlugStateSensor(RenaultBatteryDataEntity, SensorEntity):
    _attr_device_class: Any
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def icon(self) -> str: ...



from . import configure_mydevolo as configure_mydevolo
from .const import CONF_MYDEVOLO as CONF_MYDEVOLO, DEFAULT_MYDEVOLO as DEFAULT_MYDEVOLO, DOMAIN as DOMAIN, SUPPORTED_MODEL_TYPES as SUPPORTED_MODEL_TYPES
from .exceptions import CredentialsInvalid as CredentialsInvalid, UuidChanged as UuidChanged
from homeassistant import config_entries as config_entries
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.typing import DiscoveryInfoType as DiscoveryInfoType
from typing import Any

class DevoloHomeControlFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    data_schema: Any
    _reauth_entry: Any
    _url: Any
    def __init__(self) -> None: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_zeroconf(self, discovery_info: DiscoveryInfoType) -> FlowResult: ...
    async def async_step_zeroconf_confirm(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_reauth(self, user_input: dict[str, Any]) -> FlowResult: ...
    async def async_step_reauth_confirm(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def _connect_mydevolo(self, user_input: dict[str, Any]) -> FlowResult: ...
    def _show_form(self, step_id: str, errors: Union[dict[str, str], None] = ...) -> FlowResult: ...



from . import RitualsDataUpdateCoordinator as RitualsDataUpdateCoordinator
from .const import DOMAIN as DOMAIN
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from pyrituals import Diffuser as Diffuser
from typing import Any

MANUFACTURER: str
MODEL: str
MODEL2: str

class DiffuserEntity(CoordinatorEntity):
    coordinator: RitualsDataUpdateCoordinator
    _diffuser: Any
    _attr_name: Any
    _attr_unique_id: Any
    _attr_device_info: Any
    def __init__(self, diffuser: Diffuser, coordinator: RitualsDataUpdateCoordinator, entity_suffix: str) -> None: ...
    @property
    def available(self) -> bool: ...



from .const import CONF_FFMPEG_ARGUMENTS as CONF_FFMPEG_ARGUMENTS, DATA_COORDINATOR as DATA_COORDINATOR, DATA_UNDO_UPDATE_LISTENER as DATA_UNDO_UPDATE_LISTENER, DEFAULT_FFMPEG_ARGUMENTS as DEFAULT_FFMPEG_ARGUMENTS, DEFAULT_TIMEOUT as DEFAULT_TIMEOUT, DOMAIN as DOMAIN
from .coordinator import CanaryDataUpdateCoordinator as CanaryDataUpdateCoordinator
from canary.api import Api
from homeassistant.config_entries import ConfigEntry as ConfigEntry, SOURCE_IMPORT as SOURCE_IMPORT
from homeassistant.const import CONF_PASSWORD as CONF_PASSWORD, CONF_TIMEOUT as CONF_TIMEOUT, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any, Final

_LOGGER: Final[Any]
MIN_TIME_BETWEEN_UPDATES: Final[Any]
CONFIG_SCHEMA: Final[Any]
PLATFORMS: Final[list[str]]

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def _async_update_listener(hass: HomeAssistant, entry: ConfigEntry) -> None: ...
def _get_canary_api_instance(entry: ConfigEntry) -> Api: ...



import logging.handlers
from collections.abc import Coroutine
from homeassistant.const import EVENT_HOMEASSISTANT_CLOSE as EVENT_HOMEASSISTANT_CLOSE
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback, is_callback as is_callback
from typing import Any, Awaitable, Callable, overload

class HideSensitiveDataFilter(logging.Filter):
    text: Any
    def __init__(self, text: str) -> None: ...
    def filter(self, record: logging.LogRecord) -> bool: ...

class HomeAssistantQueueHandler(logging.handlers.QueueHandler):
    def handle(self, record: logging.LogRecord) -> Any: ...

def async_activate_log_queue_handler(hass: HomeAssistant) -> None: ...
def log_exception(format_err: Callable[..., Any], *args: Any) -> None: ...
@overload
def catch_log_exception(func: Callable[..., Awaitable[Any]], format_err: Callable[..., Any], *args: Any) -> Callable[..., Awaitable[None]]: ...
@overload
def catch_log_exception(func: Callable[..., Any], format_err: Callable[..., Any], *args: Any) -> Callable[..., None]: ...
def catch_log_coro_exception(target: Coroutine[Any, Any, Any], format_err: Callable[..., Any], *args: Any) -> Coroutine[Any, Any, Any]: ...
def async_create_catching_coro(target: Coroutine) -> Coroutine: ...



from . import DEVICE_CLASS_BATTERY as DEVICE_CLASS_BATTERY, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE
from homeassistant.const import ATTR_DEVICE_CLASS as ATTR_DEVICE_CLASS, ATTR_UNIT_OF_MEASUREMENT as ATTR_UNIT_OF_MEASUREMENT, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from typing import Any

def async_check_significant_change(hass: HomeAssistant, old_state: str, old_attrs: dict, new_state: str, new_attrs: dict, **kwargs: Any) -> Union[bool, None]: ...



from .const import DATA_TASK as DATA_TASK, DOMAIN as DOMAIN, PLATFORMS as PLATFORMS
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_update_options(hass: HomeAssistant, entry: ConfigEntry) -> None: ...



from homeassistant.components.sensor import PLATFORM_SCHEMA as PLATFORM_SCHEMA, SensorEntity as SensorEntity
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any

_LOGGER: Any
CONF_HOSTNAME: str
CONF_IPV6: str
CONF_RESOLVER: str
CONF_RESOLVER_IPV6: str
DEFAULT_HOSTNAME: str
DEFAULT_IPV6: bool
DEFAULT_NAME: str
DEFAULT_RESOLVER: str
DEFAULT_RESOLVER_IPV6: str
SCAN_INTERVAL: Any

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_devices: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...

class WanIpSensor(SensorEntity):
    _attr_name: Any
    hostname: Any
    resolver: Any
    querytype: Any
    def __init__(self, name: str, hostname: str, resolver: str, ipv6: bool) -> None: ...
    _attr_state: Any
    async def async_update(self) -> None: ...



from typing import Any

DOMAIN: str
LOGGER: Any
DATA_CLIENT: str
DATA_LISTENER: str
DATA_PROTECTION_WINDOW: str
DATA_UV: str
TYPE_CURRENT_OZONE_LEVEL: str
TYPE_CURRENT_UV_INDEX: str
TYPE_CURRENT_UV_LEVEL: str
TYPE_MAX_UV_INDEX: str
TYPE_PROTECTION_WINDOW: str
TYPE_SAFE_EXPOSURE_TIME_1: str
TYPE_SAFE_EXPOSURE_TIME_2: str
TYPE_SAFE_EXPOSURE_TIME_3: str
TYPE_SAFE_EXPOSURE_TIME_4: str
TYPE_SAFE_EXPOSURE_TIME_5: str
TYPE_SAFE_EXPOSURE_TIME_6: str



from .const import ATTR_OPTIONS as ATTR_OPTIONS, CONF_OPTION as CONF_OPTION, DOMAIN as DOMAIN
from homeassistant.const import CONF_CONDITION as CONF_CONDITION, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_FOR as CONF_FOR, CONF_TYPE as CONF_TYPE
from homeassistant.core import HomeAssistant as HomeAssistant, HomeAssistantError as HomeAssistantError, callback as callback
from homeassistant.helpers import condition as condition, entity_registry as entity_registry
from homeassistant.helpers.config_validation import DEVICE_CONDITION_BASE_SCHEMA as DEVICE_CONDITION_BASE_SCHEMA
from homeassistant.helpers.entity import get_capability as get_capability
from homeassistant.helpers.typing import ConfigType as ConfigType, TemplateVarsType as TemplateVarsType
from typing import Any

CONDITION_TYPES: Any
CONDITION_SCHEMA: Any

async def async_get_conditions(hass: HomeAssistant, device_id: str) -> list[dict[str, str]]: ...
def async_condition_from_config(config: ConfigType, config_validation: bool) -> condition.ConditionCheckerType: ...
async def async_get_condition_capabilities(hass: HomeAssistant, config: ConfigType) -> dict[str, Any]: ...



from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from typing import Any

def async_check_significant_change(hass: HomeAssistant, old_state: str, old_attrs: dict, new_state: str, new_attrs: dict, **kwargs: Any) -> Union[bool, None]: ...



from homeassistant.const import CONF_EVENT as CONF_EVENT, CONF_OFFSET as CONF_OFFSET, CONF_PLATFORM as CONF_PLATFORM, SUN_EVENT_SUNRISE as SUN_EVENT_SUNRISE
from homeassistant.core import HassJob as HassJob, callback as callback
from homeassistant.helpers.event import async_track_sunrise as async_track_sunrise, async_track_sunset as async_track_sunset
from typing import Any

TRIGGER_SCHEMA: Any

async def async_attach_trigger(hass, config, action, automation_info): ...



from .const import CAT_ENTITIES as CAT_ENTITIES
from .entities import ENTITY_POLICY_SCHEMA as ENTITY_POLICY_SCHEMA, compile_entities as compile_entities
from .merge import merge_policies as merge_policies
from .models import PermissionLookup as PermissionLookup
from .types import PolicyType as PolicyType
from .util import test_all as test_all
from typing import Any, Callable

POLICY_SCHEMA: Any
_LOGGER: Any

class AbstractPermissions:
    _cached_entity_func: Union[Callable[[str, str], bool], None]
    def _entity_func(self) -> Callable[[str, str], bool]: ...
    def access_all_entities(self, key: str) -> bool: ...
    def check_entity(self, entity_id: str, key: str) -> bool: ...

class PolicyPermissions(AbstractPermissions):
    _policy: Any
    _perm_lookup: Any
    def __init__(self, policy: PolicyType, perm_lookup: PermissionLookup) -> None: ...
    def access_all_entities(self, key: str) -> bool: ...
    def _entity_func(self) -> Callable[[str, str], bool]: ...
    def __eq__(self, other: Any) -> bool: ...

class _OwnerPermissions(AbstractPermissions):
    def access_all_entities(self, key: str) -> bool: ...
    def _entity_func(self) -> Callable[[str, str], bool]: ...

OwnerPermissions: Any



from homeassistant.components.automation import AutomationActionType as AutomationActionType
from homeassistant.const import CONF_EVENT_DATA as CONF_EVENT_DATA, CONF_PLATFORM as CONF_PLATFORM
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, Event as Event, HassJob as HassJob, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import template as template
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

CONF_EVENT_TYPE: str
CONF_EVENT_CONTEXT: str
TRIGGER_SCHEMA: Any

def _schema_value(value: Any) -> Any: ...
async def async_attach_trigger(hass: HomeAssistant, config: ConfigType, action: AutomationActionType, automation_info: dict[str, Any], *, platform_type: str = ...) -> CALLBACK_TYPE: ...



from . import DOMAIN as DOMAIN, SUPPORT_CLOSE as SUPPORT_CLOSE, SUPPORT_OPEN as SUPPORT_OPEN, SUPPORT_SET_POSITION as SUPPORT_SET_POSITION, SUPPORT_SET_TILT_POSITION as SUPPORT_SET_TILT_POSITION
from homeassistant.components.automation import AutomationActionType as AutomationActionType
from homeassistant.components.device_automation import DEVICE_TRIGGER_BASE_SCHEMA as DEVICE_TRIGGER_BASE_SCHEMA
from homeassistant.const import CONF_ABOVE as CONF_ABOVE, CONF_BELOW as CONF_BELOW, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_FOR as CONF_FOR, CONF_PLATFORM as CONF_PLATFORM, CONF_TYPE as CONF_TYPE, CONF_VALUE_TEMPLATE as CONF_VALUE_TEMPLATE, STATE_CLOSED as STATE_CLOSED, STATE_CLOSING as STATE_CLOSING, STATE_OPEN as STATE_OPEN, STATE_OPENING as STATE_OPENING
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, HomeAssistant as HomeAssistant
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.entity import get_supported_features as get_supported_features
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

POSITION_TRIGGER_TYPES: Any
STATE_TRIGGER_TYPES: Any
POSITION_TRIGGER_SCHEMA: Any
STATE_TRIGGER_SCHEMA: Any
TRIGGER_SCHEMA: Any

async def async_get_triggers(hass: HomeAssistant, device_id: str) -> list[dict]: ...
async def async_get_trigger_capabilities(hass: HomeAssistant, config: dict) -> dict: ...
async def async_attach_trigger(hass: HomeAssistant, config: ConfigType, action: AutomationActionType, automation_info: dict) -> CALLBACK_TYPE: ...



from . import NestSensorDevice as NestSensorDevice
from .const import DATA_NEST as DATA_NEST, DATA_NEST_CONFIG as DATA_NEST_CONFIG
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.const import CONF_MONITORED_CONDITIONS as CONF_MONITORED_CONDITIONS, CONF_SENSORS as CONF_SENSORS, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, PERCENTAGE as PERCENTAGE, STATE_OFF as STATE_OFF, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT
from typing import Any

SENSOR_TYPES: Any
TEMP_SENSOR_TYPES: Any
PROTECT_SENSOR_TYPES: Any
STRUCTURE_SENSOR_TYPES: Any
STATE_HEAT: str
STATE_COOL: str
STRUCTURE_CAMERA_SENSOR_TYPES: Any
_VALID_SENSOR_TYPES: Any
SENSOR_UNITS: Any
SENSOR_DEVICE_CLASSES: Any
VARIABLE_NAME_MAPPING: Any
VALUE_MAPPING: Any
SENSOR_TYPES_DEPRECATED: Any
DEPRECATED_WEATHER_VARS: Any
_SENSOR_TYPES_DEPRECATED: Any
_LOGGER: Any

def setup_platform(hass, config, add_entities, discovery_info: Any | None = ...) -> None: ...
async def async_setup_legacy_entry(hass, entry, async_add_entities) -> None: ...

class NestBasicSensor(NestSensorDevice, SensorEntity):
    @property
    def unit_of_measurement(self): ...
    @property
    def state(self): ...
    @property
    def device_class(self): ...
    _unit: Any
    _state: Any
    def update(self) -> None: ...

class NestTempSensor(NestSensorDevice, SensorEntity):
    @property
    def state(self): ...
    @property
    def unit_of_measurement(self): ...
    @property
    def device_class(self): ...
    _unit: Any
    _state: Any
    def update(self) -> None: ...



from typing import Any

LOGGER: Any
DOMAIN: str
DATA_CLIENT: str
DEFAULT_SCAN_INTERVAL: Any
ATTR_ALARM_DURATION: str
ATTR_ALARM_VOLUME: str
ATTR_CHIME_VOLUME: str
ATTR_ENTRY_DELAY_AWAY: str
ATTR_ENTRY_DELAY_HOME: str
ATTR_EXIT_DELAY_AWAY: str
ATTR_EXIT_DELAY_HOME: str
ATTR_LIGHT: str
ATTR_VOICE_PROMPT_VOLUME: str
VOLUMES: Any
VOLUME_STRING_MAP: Any



from typing import Any

DOMAIN: str
CONF_LOCALE: str
CONF_KAMEREON_ACCOUNT_ID: str
DEFAULT_SCAN_INTERVAL: int
PLATFORMS: Any
DEVICE_CLASS_PLUG_STATE: str
DEVICE_CLASS_CHARGE_STATE: str
DEVICE_CLASS_CHARGE_MODE: str



from .const import CONF_INSTANCE_CLIENTS as CONF_INSTANCE_CLIENTS, CONF_ON_UNLOAD as CONF_ON_UNLOAD, CONF_ROOT_CLIENT as CONF_ROOT_CLIENT, DEFAULT_NAME as DEFAULT_NAME, DOMAIN as DOMAIN, HYPERION_RELEASES_URL as HYPERION_RELEASES_URL, HYPERION_VERSION_WARN_CUTOFF as HYPERION_VERSION_WARN_CUTOFF, SIGNAL_INSTANCE_ADD as SIGNAL_INSTANCE_ADD, SIGNAL_INSTANCE_REMOVE as SIGNAL_INSTANCE_REMOVE
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_PORT as CONF_PORT, CONF_TOKEN as CONF_TOKEN
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryAuthFailed as ConfigEntryAuthFailed, ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect, async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.typing import ConfigType as ConfigType
from hyperion import client
from typing import Any, Callable

PLATFORMS: Any
_LOGGER: Any

def get_hyperion_unique_id(server_id: str, instance: int, name: str) -> str: ...
def get_hyperion_device_id(server_id: str, instance: int) -> str: ...
def split_hyperion_unique_id(unique_id: str) -> Union[tuple[str, int, str], None]: ...
def create_hyperion_client(*args: Any, **kwargs: Any) -> client.HyperionClient: ...
async def async_create_connect_hyperion_client(*args: Any, **kwargs: Any) -> Union[client.HyperionClient, None]: ...
async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
def listen_for_instance_updates(hass: HomeAssistant, config_entry: ConfigEntry, add_func: Callable, remove_func: Callable) -> None: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def _async_entry_updated(hass: HomeAssistant, config_entry: ConfigEntry) -> None: ...
async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...



import voluptuous as vol
from . import MULTI_FACTOR_AUTH_MODULES as MULTI_FACTOR_AUTH_MODULES, MULTI_FACTOR_AUTH_MODULE_SCHEMA as MULTI_FACTOR_AUTH_MODULE_SCHEMA, MultiFactorAuthModule as MultiFactorAuthModule, SetupFlow as SetupFlow
from homeassistant.const import CONF_EXCLUDE as CONF_EXCLUDE, CONF_INCLUDE as CONF_INCLUDE
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.exceptions import ServiceNotFound as ServiceNotFound
from typing import Any, Dict

REQUIREMENTS: Any
CONF_MESSAGE: str
CONFIG_SCHEMA: Any
STORAGE_VERSION: int
STORAGE_KEY: str
STORAGE_USERS: str
STORAGE_USER_ID: str
INPUT_FIELD_CODE: str
_LOGGER: Any

def _generate_secret() -> str: ...
def _generate_random() -> int: ...
def _generate_otp(secret: str, count: int) -> str: ...
def _verify_otp(secret: str, otp: str, count: int) -> bool: ...

class NotifySetting:
    secret: str
    counter: int
    notify_service: Union[str, None]
    target: Union[str, None]
    def __init__(self, secret, counter, notify_service, target) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
_UsersDict = Dict[str, NotifySetting]

class NotifyAuthModule(MultiFactorAuthModule):
    DEFAULT_TITLE: str
    _user_settings: Any
    _user_store: Any
    _include: Any
    _exclude: Any
    _message_template: Any
    _init_lock: Any
    def __init__(self, hass: HomeAssistant, config: dict[str, Any]) -> None: ...
    @property
    def input_schema(self) -> vol.Schema: ...
    async def _async_load(self) -> None: ...
    async def _async_save(self) -> None: ...
    def aync_get_available_notify_services(self) -> list[str]: ...
    async def async_setup_flow(self, user_id: str) -> SetupFlow: ...
    async def async_setup_user(self, user_id: str, setup_data: Any) -> Any: ...
    async def async_depose_user(self, user_id: str) -> None: ...
    async def async_is_user_setup(self, user_id: str) -> bool: ...
    async def async_validate(self, user_id: str, user_input: dict[str, Any]) -> bool: ...
    async def async_initialize_login_mfa_step(self, user_id: str) -> None: ...
    async def async_notify_user(self, user_id: str, code: str) -> None: ...
    async def async_notify(self, code: str, notify_service: str, target: Union[str, None] = ...) -> None: ...

class NotifySetupFlow(SetupFlow):
    _auth_module: Any
    _available_notify_services: Any
    _secret: Any
    _count: Any
    _notify_service: Any
    _target: Any
    def __init__(self, auth_module: NotifyAuthModule, setup_schema: vol.Schema, user_id: str, available_notify_services: list[str]) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...
    async def async_step_setup(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...



from . import BraviaTVCoordinator as BraviaTVCoordinator
from .const import ATTR_MANUFACTURER as ATTR_MANUFACTURER, DEFAULT_NAME as DEFAULT_NAME, DOMAIN as DOMAIN
from collections.abc import Iterable
from homeassistant.components.remote import ATTR_NUM_REPEATS as ATTR_NUM_REPEATS, RemoteEntity as RemoteEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from typing import Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class BraviaTVRemote(CoordinatorEntity, RemoteEntity):
    coordinator: BraviaTVCoordinator
    _attr_device_info: Any
    _attr_name: Any
    _attr_unique_id: Any
    def __init__(self, coordinator: BraviaTVCoordinator, name: str, unique_id: str, device_info: DeviceInfo) -> None: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def async_send_command(self, command: Iterable[str], **kwargs: Any) -> None: ...



from .const import ATTR_API_ADVICE as ATTR_API_ADVICE, ATTR_API_CAQI as ATTR_API_CAQI, ATTR_API_CAQI_DESCRIPTION as ATTR_API_CAQI_DESCRIPTION, ATTR_API_CAQI_LEVEL as ATTR_API_CAQI_LEVEL, CONF_USE_NEAREST as CONF_USE_NEAREST, DOMAIN as DOMAIN, MAX_UPDATE_INTERVAL as MAX_UPDATE_INTERVAL, MIN_UPDATE_INTERVAL as MIN_UPDATE_INTERVAL, NO_AIRLY_SENSORS as NO_AIRLY_SENSORS
from aiohttp import ClientSession as ClientSession
from datetime import timedelta
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_API_KEY as CONF_API_KEY, CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from homeassistant.helpers.device_registry import async_get_registry as async_get_registry
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from typing import Any

PLATFORMS: Any
_LOGGER: Any

def set_update_interval(instances_count: int, requests_remaining: int) -> timedelta: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class AirlyDataUpdateCoordinator(DataUpdateCoordinator):
    latitude: Any
    longitude: Any
    airly: Any
    use_nearest: Any
    def __init__(self, hass: HomeAssistant, session: ClientSession, api_key: str, latitude: float, longitude: float, update_interval: timedelta, use_nearest: bool) -> None: ...
    update_interval: Any
    async def _async_update_data(self) -> dict[str, Union[str, float, int]]: ...



from .const import DiscoveryInfo as DiscoveryInfo, MYSENSORS_DISCOVERY as MYSENSORS_DISCOVERY, SensorType as SensorType
from .device import MySensorsDevice as MySensorsDevice
from .helpers import on_unload as on_unload
from homeassistant.components import mysensors as mysensors
from homeassistant.components.light import ATTR_BRIGHTNESS as ATTR_BRIGHTNESS, ATTR_HS_COLOR as ATTR_HS_COLOR, ATTR_WHITE_VALUE as ATTR_WHITE_VALUE, DOMAIN as DOMAIN, LightEntity as LightEntity, SUPPORT_BRIGHTNESS as SUPPORT_BRIGHTNESS, SUPPORT_COLOR as SUPPORT_COLOR, SUPPORT_WHITE_VALUE as SUPPORT_WHITE_VALUE
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import STATE_OFF as STATE_OFF, STATE_ON as STATE_ON
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.color import rgb_hex_to_rgb_list as rgb_hex_to_rgb_list
from typing import Any

SUPPORT_MYSENSORS_RGBW: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class MySensorsLight(mysensors.device.MySensorsEntity, LightEntity):
    _state: Any
    _brightness: Any
    _hs: Any
    _white: Any
    def __init__(self, *args: Any) -> None: ...
    @property
    def brightness(self) -> Union[int, None]: ...
    @property
    def hs_color(self) -> Union[tuple[int, int], None]: ...
    @property
    def white_value(self) -> Union[int, None]: ...
    @property
    def is_on(self) -> bool: ...
    def _turn_on_light(self) -> None: ...
    def _turn_on_dimmer(self, **kwargs: Any) -> None: ...
    def _turn_on_rgb_and_w(self, hex_template: str, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    def _async_update_light(self) -> None: ...
    def _async_update_dimmer(self) -> None: ...
    def _async_update_rgb_or_w(self) -> None: ...

class MySensorsLightDimmer(MySensorsLight):
    @property
    def supported_features(self) -> int: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_update(self) -> None: ...

class MySensorsLightRGB(MySensorsLight):
    @property
    def supported_features(self) -> int: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_update(self) -> None: ...

class MySensorsLightRGBW(MySensorsLightRGB):
    @property
    def supported_features(self) -> int: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...



import pyatmo
from .const import DATA_HANDLER as DATA_HANDLER, DOMAIN as DOMAIN, EVENT_TYPE_LIGHT_MODE as EVENT_TYPE_LIGHT_MODE, MANUFACTURER as MANUFACTURER, SIGNAL_NAME as SIGNAL_NAME, WEBHOOK_LIGHT_MODE as WEBHOOK_LIGHT_MODE, WEBHOOK_PUSH_TYPE as WEBHOOK_PUSH_TYPE
from .data_handler import CAMERA_DATA_CLASS_NAME as CAMERA_DATA_CLASS_NAME, NetatmoDataHandler as NetatmoDataHandler
from .netatmo_entity_base import NetatmoBase as NetatmoBase
from homeassistant.components.light import LightEntity as LightEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import PlatformNotReady as PlatformNotReady
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

_LOGGER: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class NetatmoLight(NetatmoBase, LightEntity):
    _id: Any
    _home_id: Any
    _model: Any
    _device_name: Any
    _attr_name: Any
    _is_on: bool
    _attr_unique_id: Any
    def __init__(self, data_handler: NetatmoDataHandler, camera_id: str, camera_type: str, home_id: str) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    def handle_event(self, event: dict) -> None: ...
    @property
    def _data(self) -> pyatmo.AsyncCameraData: ...
    @property
    def available(self) -> bool: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: dict) -> None: ...
    async def async_turn_off(self, **kwargs: dict) -> None: ...
    def async_update_callback(self) -> None: ...



from homeassistant.components.sensor import SensorEntityDescription as SensorEntityDescription
from typing import Callable

class GiosSensorEntityDescription(SensorEntityDescription):
    value: Union[Callable, None]



from homeassistant.components.device_automation import DEVICE_TRIGGER_BASE_SCHEMA as DEVICE_TRIGGER_BASE_SCHEMA, async_get_device_automation_platform as async_get_device_automation_platform
from homeassistant.const import CONF_DOMAIN as CONF_DOMAIN
from typing import Any

TRIGGER_SCHEMA: Any

async def async_validate_trigger_config(hass, config): ...
async def async_attach_trigger(hass, config, action, automation_info): ...



import logging
from .debounce import Debouncer as Debouncer
from collections.abc import Awaitable
from datetime import datetime, timedelta
from homeassistant import config_entries as config_entries
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, Event as Event, HassJob as HassJob, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryAuthFailed as ConfigEntryAuthFailed, ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import entity as entity, event as event
from homeassistant.util.dt import utcnow as utcnow
from typing import Any, Callable, TypeVar

REQUEST_REFRESH_DEFAULT_COOLDOWN: int
REQUEST_REFRESH_DEFAULT_IMMEDIATE: bool
T = TypeVar('T')

class UpdateFailed(Exception): ...

class DataUpdateCoordinator:
    hass: Any
    logger: Any
    name: Any
    update_method: Any
    update_interval: Any
    config_entry: Any
    data: Any
    _listeners: Any
    _job: Any
    _unsub_refresh: Any
    _request_refresh_task: Any
    last_update_success: bool
    last_exception: Any
    _debounced_refresh: Any
    def __init__(self, hass: HomeAssistant, logger: logging.Logger, name: str, *, update_interval: Union[timedelta, None] = ..., update_method: Union[Callable[[], Awaitable[T]], None] = ..., request_refresh_debouncer: Union[Debouncer, None] = ...) -> None: ...
    def async_add_listener(self, update_callback: CALLBACK_TYPE) -> Callable[[], None]: ...
    def async_remove_listener(self, update_callback: CALLBACK_TYPE) -> None: ...
    def _schedule_refresh(self) -> None: ...
    async def _handle_refresh_interval(self, _now: datetime) -> None: ...
    async def async_request_refresh(self) -> None: ...
    async def _async_update_data(self) -> T: ...
    async def async_config_entry_first_refresh(self) -> None: ...
    async def async_refresh(self) -> None: ...
    async def _async_refresh(self, log_failures: bool = ..., raise_on_auth_failed: bool = ..., scheduled: bool = ...) -> None: ...
    def async_set_updated_data(self, data: T) -> None: ...
    def _async_stop_refresh(self, _: Event) -> None: ...

class CoordinatorEntity(entity.Entity):
    coordinator: Any
    def __init__(self, coordinator: DataUpdateCoordinator[T]) -> None: ...
    @property
    def should_poll(self) -> bool: ...
    @property
    def available(self) -> bool: ...
    async def async_added_to_hass(self) -> None: ...
    def _handle_coordinator_update(self) -> None: ...
    async def async_update(self) -> None: ...



from .const import ATTR_LAST_DATA as ATTR_LAST_DATA, ATTR_MONITORED_CONDITIONS as ATTR_MONITORED_CONDITIONS, CONF_APP_KEY as CONF_APP_KEY, DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN, LOGGER as LOGGER
from aioambient import Client
from homeassistant.components.binary_sensor import DEVICE_CLASS_CONNECTIVITY as DEVICE_CLASS_CONNECTIVITY
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_LOCATION as ATTR_LOCATION, ATTR_NAME as ATTR_NAME, CONCENTRATION_MICROGRAMS_PER_CUBIC_METER as CONCENTRATION_MICROGRAMS_PER_CUBIC_METER, CONCENTRATION_PARTS_PER_MILLION as CONCENTRATION_PARTS_PER_MILLION, CONF_API_KEY as CONF_API_KEY, DEGREE as DEGREE, DEVICE_CLASS_BATTERY as DEVICE_CLASS_BATTERY, DEVICE_CLASS_CO2 as DEVICE_CLASS_CO2, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_ILLUMINANCE as DEVICE_CLASS_ILLUMINANCE, DEVICE_CLASS_PRESSURE as DEVICE_CLASS_PRESSURE, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, DEVICE_CLASS_TIMESTAMP as DEVICE_CLASS_TIMESTAMP, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP, IRRADIATION_WATTS_PER_SQUARE_METER as IRRADIATION_WATTS_PER_SQUARE_METER, LIGHT_LUX as LIGHT_LUX, PERCENTAGE as PERCENTAGE, PRECIPITATION_INCHES as PRECIPITATION_INCHES, PRECIPITATION_INCHES_PER_HOUR as PRECIPITATION_INCHES_PER_HOUR, PRESSURE_INHG as PRESSURE_INHG, SPEED_MILES_PER_HOUR as SPEED_MILES_PER_HOUR, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import aiohttp_client as aiohttp_client
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect, async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.event import async_call_later as async_call_later
from typing import Any

PLATFORMS: Any
DATA_CONFIG: str
DEFAULT_SOCKET_MIN_RETRY: int
TYPE_24HOURRAININ: str
TYPE_BAROMABSIN: str
TYPE_BAROMRELIN: str
TYPE_BATT1: str
TYPE_BATT10: str
TYPE_BATT2: str
TYPE_BATT3: str
TYPE_BATT4: str
TYPE_BATT5: str
TYPE_BATT6: str
TYPE_BATT7: str
TYPE_BATT8: str
TYPE_BATT9: str
TYPE_BATT_CO2: str
TYPE_BATTOUT: str
TYPE_CO2: str
TYPE_DAILYRAININ: str
TYPE_DEWPOINT: str
TYPE_EVENTRAININ: str
TYPE_FEELSLIKE: str
TYPE_HOURLYRAININ: str
TYPE_HUMIDITY: str
TYPE_HUMIDITY1: str
TYPE_HUMIDITY10: str
TYPE_HUMIDITY2: str
TYPE_HUMIDITY3: str
TYPE_HUMIDITY4: str
TYPE_HUMIDITY5: str
TYPE_HUMIDITY6: str
TYPE_HUMIDITY7: str
TYPE_HUMIDITY8: str
TYPE_HUMIDITY9: str
TYPE_HUMIDITYIN: str
TYPE_LASTRAIN: str
TYPE_MAXDAILYGUST: str
TYPE_MONTHLYRAININ: str
TYPE_PM25: str
TYPE_PM25_24H: str
TYPE_PM25_BATT: str
TYPE_PM25_IN: str
TYPE_PM25_IN_24H: str
TYPE_PM25IN_BATT: str
TYPE_RELAY1: str
TYPE_RELAY10: str
TYPE_RELAY2: str
TYPE_RELAY3: str
TYPE_RELAY4: str
TYPE_RELAY5: str
TYPE_RELAY6: str
TYPE_RELAY7: str
TYPE_RELAY8: str
TYPE_RELAY9: str
TYPE_SOILHUM1: str
TYPE_SOILHUM10: str
TYPE_SOILHUM2: str
TYPE_SOILHUM3: str
TYPE_SOILHUM4: str
TYPE_SOILHUM5: str
TYPE_SOILHUM6: str
TYPE_SOILHUM7: str
TYPE_SOILHUM8: str
TYPE_SOILHUM9: str
TYPE_SOILTEMP1F: str
TYPE_SOILTEMP10F: str
TYPE_SOILTEMP2F: str
TYPE_SOILTEMP3F: str
TYPE_SOILTEMP4F: str
TYPE_SOILTEMP5F: str
TYPE_SOILTEMP6F: str
TYPE_SOILTEMP7F: str
TYPE_SOILTEMP8F: str
TYPE_SOILTEMP9F: str
TYPE_SOLARRADIATION: str
TYPE_SOLARRADIATION_LX: str
TYPE_TEMP10F: str
TYPE_TEMP1F: str
TYPE_TEMP2F: str
TYPE_TEMP3F: str
TYPE_TEMP4F: str
TYPE_TEMP5F: str
TYPE_TEMP6F: str
TYPE_TEMP7F: str
TYPE_TEMP8F: str
TYPE_TEMP9F: str
TYPE_TEMPF: str
TYPE_TEMPINF: str
TYPE_TOTALRAININ: str
TYPE_UV: str
TYPE_WEEKLYRAININ: str
TYPE_WINDDIR: str
TYPE_WINDDIR_AVG10M: str
TYPE_WINDDIR_AVG2M: str
TYPE_WINDGUSTDIR: str
TYPE_WINDGUSTMPH: str
TYPE_WINDSPDMPH_AVG10M: str
TYPE_WINDSPDMPH_AVG2M: str
TYPE_WINDSPEEDMPH: str
TYPE_YEARLYRAININ: str
SENSOR_TYPES: Any
CONFIG_SCHEMA: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...
async def async_migrate_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...

class AmbientStation:
    _config_entry: Any
    _entry_setup_complete: bool
    _hass: Any
    _ws_reconnect_delay: Any
    client: Any
    stations: Any
    def __init__(self, hass: HomeAssistant, config_entry: ConfigEntry, client: Client) -> None: ...
    async def _attempt_connect(self) -> None: ...
    async def ws_connect(self) -> None: ...
    async def ws_disconnect(self) -> None: ...

class AmbientWeatherEntity(Entity):
    _ambient: Any
    _attr_device_class: Any
    _attr_device_info: Any
    _attr_name: Any
    _attr_should_poll: bool
    _attr_unique_id: Any
    _mac_address: Any
    _sensor_type: Any
    def __init__(self, ambient: AmbientStation, mac_address: str, station_name: str, sensor_type: str, sensor_name: str, device_class: Union[str, None]) -> None: ...
    _attr_available: Any
    async def async_added_to_hass(self) -> None: ...
    def update_from_latest_data(self) -> None: ...



from .const import API_TIMEOUT as API_TIMEOUT, CONF_STATION_ID as CONF_STATION_ID, DOMAIN as DOMAIN, SCAN_INTERVAL as SCAN_INTERVAL
from aiohttp import ClientSession as ClientSession
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from homeassistant.helpers.device_registry import async_get_registry as async_get_registry
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from typing import Any

_LOGGER: Any
PLATFORMS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class GiosDataUpdateCoordinator(DataUpdateCoordinator):
    gios: Any
    def __init__(self, hass: HomeAssistant, session: ClientSession, station_id: int) -> None: ...
    async def _async_update_data(self) -> dict[str, Any]: ...



from typing import Mapping, Union

ValueType = Union[Mapping[str, bool], bool, None]
SubCategoryDict = Mapping[str, ValueType]
SubCategoryType = Union[SubCategoryDict, bool, None]
CategoryType = Union[Mapping[str, SubCategoryType], Mapping[str, ValueType], bool, None]
PolicyType = Mapping[str, CategoryType]



from . import const as const
from homeassistant.core import Event as Event
from homeassistant.util.json import find_paths_unserializable_data as find_paths_unserializable_data, format_unserializable_data as format_unserializable_data
from homeassistant.util.yaml.loader import JSON_TYPE as JSON_TYPE
from typing import Any, Final

_LOGGER: Final[Any]
MINIMAL_MESSAGE_SCHEMA: Final[Any]
BASE_COMMAND_MESSAGE_SCHEMA: Final[Any]
IDEN_TEMPLATE: Final[str]
IDEN_JSON_TEMPLATE: Final[str]

def result_message(iden: int, result: Any = ...) -> dict[str, Any]: ...
def error_message(iden: Union[int, None], code: str, message: str) -> dict[str, Any]: ...
def event_message(iden: JSON_TYPE, event: Any) -> dict[str, Any]: ...
def cached_event_message(iden: int, event: Event) -> str: ...
def _cached_event_message(event: Event) -> str: ...
def message_to_json(message: dict[str, Any]) -> str: ...



from .const import DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo
from .entity import ZWaveBaseEntity as ZWaveBaseEntity
from homeassistant.components.fan import FanEntity as FanEntity, SUPPORT_SET_SPEED as SUPPORT_SET_SPEED
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.percentage import int_states_in_range as int_states_in_range, percentage_to_ranged_value as percentage_to_ranged_value, ranged_value_to_percentage as ranged_value_to_percentage
from typing import Any

SUPPORTED_FEATURES = SUPPORT_SET_SPEED
SPEED_RANGE: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class ZwaveFan(ZWaveBaseEntity, FanEntity):
    async def async_set_percentage(self, percentage: Union[int, None]) -> None: ...
    async def async_turn_on(self, speed: Union[str, None] = ..., percentage: Union[int, None] = ..., preset_mode: Union[str, None] = ..., **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    @property
    def is_on(self) -> Union[bool, None]: ...
    @property
    def percentage(self) -> Union[int, None]: ...
    @property
    def speed_count(self) -> int: ...
    @property
    def supported_features(self) -> int: ...



from . import const as const
from .helpers import async_get_node_from_device_id as async_get_node_from_device_id, async_get_node_from_entity_id as async_get_node_from_entity_id
from homeassistant.const import ATTR_DEVICE_ID as ATTR_DEVICE_ID, ATTR_ENTITY_ID as ATTR_ENTITY_ID
from homeassistant.core import HomeAssistant as HomeAssistant, ServiceCall as ServiceCall, callback as callback
from homeassistant.helpers.device_registry import DeviceRegistry as DeviceRegistry
from homeassistant.helpers.dispatcher import async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.entity_registry import EntityRegistry as EntityRegistry
from typing import Any

_LOGGER: Any

def parameter_name_does_not_need_bitmask(val: dict[str, Union[int, str, list[str]]]) -> dict[str, Union[int, str, list[str]]]: ...
def broadcast_command(val: dict[str, Any]) -> dict[str, Any]: ...

BITMASK_SCHEMA: Any
VALUE_SCHEMA: Any

class ZWaveServices:
    _hass: Any
    _ent_reg: Any
    _dev_reg: Any
    def __init__(self, hass: HomeAssistant, ent_reg: EntityRegistry, dev_reg: DeviceRegistry) -> None: ...
    def async_register(self) -> None: ...
    async def async_set_config_parameter(self, service: ServiceCall) -> None: ...
    async def async_bulk_set_partial_config_parameters(self, service: ServiceCall) -> None: ...
    async def async_poll_value(self, service: ServiceCall) -> None: ...
    async def async_set_value(self, service: ServiceCall) -> None: ...
    async def async_multicast_set_value(self, service: ServiceCall) -> None: ...
    async def async_ping(self, service: ServiceCall) -> None: ...



import voluptuous as vol
from . import const as const, messages as messages
from .connection import ActiveConnection as ActiveConnection
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import Unauthorized as Unauthorized
from typing import Any, Callable

async def _handle_async_response(func: const.AsyncWebSocketCommandHandler, hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]) -> None: ...
def async_response(func: const.AsyncWebSocketCommandHandler) -> const.WebSocketCommandHandler: ...
def require_admin(func: const.WebSocketCommandHandler) -> const.WebSocketCommandHandler: ...
def ws_require_user(only_owner: bool = ..., only_system_user: bool = ..., allow_system_user: bool = ..., only_active_user: bool = ..., only_inactive_user: bool = ...) -> Callable[[const.WebSocketCommandHandler], const.WebSocketCommandHandler]: ...
def websocket_command(schema: dict[vol.Marker, Any]) -> Callable[[const.WebSocketCommandHandler], const.WebSocketCommandHandler]: ...



from . import RainMachineEntity as RainMachineEntity
from .const import DATA_CONTROLLER as DATA_CONTROLLER, DATA_COORDINATOR as DATA_COORDINATOR, DATA_PROVISION_SETTINGS as DATA_PROVISION_SETTINGS, DATA_RESTRICTIONS_CURRENT as DATA_RESTRICTIONS_CURRENT, DATA_RESTRICTIONS_UNIVERSAL as DATA_RESTRICTIONS_UNIVERSAL, DOMAIN as DOMAIN
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from regenmaschine.controller import Controller as Controller
from typing import Any

TYPE_FLOW_SENSOR: str
TYPE_FREEZE: str
TYPE_FREEZE_PROTECTION: str
TYPE_HOT_DAYS: str
TYPE_HOURLY: str
TYPE_MONTH: str
TYPE_RAINDELAY: str
TYPE_RAINSENSOR: str
TYPE_WEEKDAY: str
BINARY_SENSORS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class RainMachineBinarySensor(RainMachineEntity, BinarySensorEntity):
    _attr_entity_registry_enabled_default: Any
    _attr_icon: Any
    _attr_name: Any
    def __init__(self, coordinator: DataUpdateCoordinator, controller: Controller, sensor_type: str, name: str, icon: str, enabled_by_default: bool) -> None: ...

class CurrentRestrictionsBinarySensor(RainMachineBinarySensor):
    _attr_is_on: Any
    def update_from_latest_data(self) -> None: ...

class ProvisionSettingsBinarySensor(RainMachineBinarySensor):
    _attr_is_on: Any
    def update_from_latest_data(self) -> None: ...

class UniversalRestrictionsBinarySensor(RainMachineBinarySensor):
    _attr_is_on: Any
    def update_from_latest_data(self) -> None: ...



from . import RainMachineEntity as RainMachineEntity
from .const import DATA_CONTROLLER as DATA_CONTROLLER, DATA_COORDINATOR as DATA_COORDINATOR, DATA_PROVISION_SETTINGS as DATA_PROVISION_SETTINGS, DATA_RESTRICTIONS_UNIVERSAL as DATA_RESTRICTIONS_UNIVERSAL, DOMAIN as DOMAIN
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, TEMP_CELSIUS as TEMP_CELSIUS, VOLUME_CUBIC_METERS as VOLUME_CUBIC_METERS
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from regenmaschine.controller import Controller as Controller
from typing import Any

TYPE_FLOW_SENSOR_CLICK_M3: str
TYPE_FLOW_SENSOR_CONSUMED_LITERS: str
TYPE_FLOW_SENSOR_START_INDEX: str
TYPE_FLOW_SENSOR_WATERING_CLICKS: str
TYPE_FREEZE_TEMP: str
SENSORS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class RainMachineSensor(RainMachineEntity, SensorEntity):
    _attr_device_class: Any
    _attr_entity_registry_enabled_default: Any
    _attr_icon: Any
    _attr_name: Any
    _attr_unit_of_measurement: Any
    def __init__(self, coordinator: DataUpdateCoordinator, controller: Controller, sensor_type: str, name: str, icon: str, unit: str, device_class: str, enabled_by_default: bool) -> None: ...

class ProvisionSettingsSensor(RainMachineSensor):
    _attr_state: Any
    def update_from_latest_data(self) -> None: ...

class UniversalRestrictionsSensor(RainMachineSensor):
    _attr_state: Any
    def update_from_latest_data(self) -> None: ...



from homeassistant.const import ATTR_LATITUDE as ATTR_LATITUDE, ATTR_LONGITUDE as ATTR_LONGITUDE, CONF_DEVICES as CONF_DEVICES, CONF_UNIT_OF_MEASUREMENT as CONF_UNIT_OF_MEASUREMENT, CONF_ZONE as CONF_ZONE, LENGTH_FEET as LENGTH_FEET, LENGTH_KILOMETERS as LENGTH_KILOMETERS, LENGTH_METERS as LENGTH_METERS, LENGTH_MILES as LENGTH_MILES, LENGTH_YARD as LENGTH_YARD
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.event import track_state_change as track_state_change
from homeassistant.util.distance import convert as convert
from homeassistant.util.location import distance as distance
from typing import Any

_LOGGER: Any
ATTR_DIR_OF_TRAVEL: str
ATTR_DIST_FROM: str
ATTR_NEAREST: str
CONF_IGNORED_ZONES: str
CONF_TOLERANCE: str
DEFAULT_DIR_OF_TRAVEL: str
DEFAULT_DIST_TO_ZONE: str
DEFAULT_NEAREST: str
DEFAULT_PROXIMITY_ZONE: str
DEFAULT_TOLERANCE: int
DOMAIN: str
UNITS: Any
ZONE_SCHEMA: Any
CONFIG_SCHEMA: Any

def setup_proximity_component(hass, name, config): ...
def setup(hass, config): ...

class Proximity(Entity):
    hass: Any
    friendly_name: Any
    dist_to: Any
    dir_of_travel: Any
    nearest: Any
    ignored_zones: Any
    proximity_devices: Any
    tolerance: Any
    proximity_zone: Any
    _unit_of_measurement: Any
    def __init__(self, hass, zone_friendly_name, dist_to, dir_of_travel, nearest, ignored_zones, proximity_devices, tolerance, proximity_zone, unit_of_measurement) -> None: ...
    @property
    def name(self): ...
    @property
    def state(self): ...
    @property
    def unit_of_measurement(self): ...
    @property
    def extra_state_attributes(self): ...
    def check_proximity_state_change(self, entity, old_state, new_state) -> None: ...



from .const import DOMAIN as DOMAIN, SENSOR as SENSOR
from homeassistant.components.network import async_get_source_ip as async_get_source_ip
from homeassistant.components.network.const import PUBLIC_TARGET_IP as PUBLIC_TARGET_IP
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class IPSensor(SensorEntity):
    _attr_unique_id: Any
    _attr_icon: str
    _attr_name: Any
    def __init__(self, name: str) -> None: ...
    _attr_state: Any
    async def async_update(self) -> None: ...



from . import SwitcherDeviceWrapper as SwitcherDeviceWrapper
from .const import SIGNAL_DEVICE_ADD as SIGNAL_DEVICE_ADD
from homeassistant.components.sensor import DEVICE_CLASS_CURRENT as DEVICE_CLASS_CURRENT, DEVICE_CLASS_POWER as DEVICE_CLASS_POWER, STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT, SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ELECTRIC_CURRENT_AMPERE as ELECTRIC_CURRENT_AMPERE, POWER_WATT as POWER_WATT
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import device_registry as device_registry
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import StateType as StateType
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from typing import Any

class AttributeDescription:
    name: str
    icon: Union[str, None]
    unit: Union[str, None]
    device_class: Union[str, None]
    state_class: Union[str, None]
    default_enabled: bool

POWER_SENSORS: Any
TIME_SENSORS: Any
POWER_PLUG_SENSORS = POWER_SENSORS
WATER_HEATER_SENSORS: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class SwitcherSensorEntity(CoordinatorEntity, SensorEntity):
    wrapper: Any
    attribute: Any
    _attr_name: Any
    _attr_icon: Any
    _attr_unit_of_measurement: Any
    _attr_device_class: Any
    _attr_entity_registry_enabled_default: Any
    _attr_unique_id: Any
    _attr_device_info: Any
    def __init__(self, wrapper: SwitcherDeviceWrapper, attribute: str, description: AttributeDescription) -> None: ...
    @property
    def state(self) -> StateType: ...



from homeassistant.components.frontend import EVENT_PANELS_UPDATED as EVENT_PANELS_UPDATED
from homeassistant.components.lovelace.const import EVENT_LOVELACE_UPDATED as EVENT_LOVELACE_UPDATED
from homeassistant.components.persistent_notification import EVENT_PERSISTENT_NOTIFICATIONS_UPDATED as EVENT_PERSISTENT_NOTIFICATIONS_UPDATED
from homeassistant.const import EVENT_COMPONENT_LOADED as EVENT_COMPONENT_LOADED, EVENT_CORE_CONFIG_UPDATE as EVENT_CORE_CONFIG_UPDATE, EVENT_SERVICE_REGISTERED as EVENT_SERVICE_REGISTERED, EVENT_SERVICE_REMOVED as EVENT_SERVICE_REMOVED, EVENT_STATE_CHANGED as EVENT_STATE_CHANGED, EVENT_THEMES_UPDATED as EVENT_THEMES_UPDATED
from homeassistant.helpers.area_registry import EVENT_AREA_REGISTRY_UPDATED as EVENT_AREA_REGISTRY_UPDATED
from homeassistant.helpers.device_registry import EVENT_DEVICE_REGISTRY_UPDATED as EVENT_DEVICE_REGISTRY_UPDATED
from homeassistant.helpers.entity_registry import EVENT_ENTITY_REGISTRY_UPDATED as EVENT_ENTITY_REGISTRY_UPDATED
from typing import Final

SUBSCRIBE_ALLOWLIST: Final[set[str]]



from .const import CONF_CLIMATES as CONF_CLIMATES, CONF_CONNECTIONS as CONF_CONNECTIONS, CONF_DIM_MODE as CONF_DIM_MODE, CONF_DOMAIN_DATA as CONF_DOMAIN_DATA, CONF_HARDWARE_SERIAL as CONF_HARDWARE_SERIAL, CONF_HARDWARE_TYPE as CONF_HARDWARE_TYPE, CONF_RESOURCE as CONF_RESOURCE, CONF_SCENES as CONF_SCENES, CONF_SK_NUM_TRIES as CONF_SK_NUM_TRIES, CONF_SOFTWARE_SERIAL as CONF_SOFTWARE_SERIAL, CONNECTION as CONNECTION, DEFAULT_NAME as DEFAULT_NAME, DOMAIN as DOMAIN
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_ADDRESS as CONF_ADDRESS, CONF_BINARY_SENSORS as CONF_BINARY_SENSORS, CONF_COVERS as CONF_COVERS, CONF_DEVICES as CONF_DEVICES, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITIES as CONF_ENTITIES, CONF_HOST as CONF_HOST, CONF_IP_ADDRESS as CONF_IP_ADDRESS, CONF_LIGHTS as CONF_LIGHTS, CONF_NAME as CONF_NAME, CONF_PASSWORD as CONF_PASSWORD, CONF_PORT as CONF_PORT, CONF_SENSORS as CONF_SENSORS, CONF_SWITCHES as CONF_SWITCHES, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any, Tuple

AddressType = Tuple[int, int, bool]
DeviceConnectionType: Any
InputType: Any
PATTERN_ADDRESS: Any
DOMAIN_LOOKUP: Any

def get_device_connection(hass: HomeAssistant, address: AddressType, config_entry: ConfigEntry) -> Union[DeviceConnectionType, None]: ...
def get_resource(domain_name: str, domain_data: ConfigType) -> str: ...
def generate_unique_id(address: AddressType) -> str: ...
def import_lcn_config(lcn_config: ConfigType) -> list[ConfigType]: ...
def has_unique_host_names(hosts: list[ConfigType]) -> list[ConfigType]: ...
def is_address(value: str) -> tuple[AddressType, str]: ...
def is_states_string(states_string: str) -> list[str]: ...



from .const import DOMAIN as DOMAIN, PRINTER_TYPES as PRINTER_TYPES
from .utils import get_snmp_engine as get_snmp_engine
from homeassistant import config_entries as config_entries, exceptions as exceptions
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_TYPE as CONF_TYPE
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.typing import DiscoveryInfoType as DiscoveryInfoType
from typing import Any

DATA_SCHEMA: Any

def host_valid(host: str) -> bool: ...

class BrotherConfigFlow(config_entries.ConfigFlow):
    VERSION: int
    brother: Any
    host: Any
    def __init__(self) -> None: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_zeroconf(self, discovery_info: DiscoveryInfoType) -> FlowResult: ...
    async def async_step_zeroconf_confirm(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...

class InvalidHost(exceptions.HomeAssistantError): ...



from homeassistant.const import LENGTH as LENGTH, LENGTH_CENTIMETERS as LENGTH_CENTIMETERS, LENGTH_FEET as LENGTH_FEET, LENGTH_INCHES as LENGTH_INCHES, LENGTH_KILOMETERS as LENGTH_KILOMETERS, LENGTH_METERS as LENGTH_METERS, LENGTH_MILES as LENGTH_MILES, LENGTH_MILLIMETERS as LENGTH_MILLIMETERS, LENGTH_YARD as LENGTH_YARD, UNIT_NOT_RECOGNIZED_TEMPLATE as UNIT_NOT_RECOGNIZED_TEMPLATE
from typing import Callable

VALID_UNITS: tuple[str, ...]
TO_METERS: dict[str, Callable[[float], float]]
METERS_TO: dict[str, Callable[[float], float]]

def convert(value: float, unit_1: str, unit_2: str) -> float: ...



from .model import AccuWeatherSensorDescription as AccuWeatherSensorDescription
from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT
from homeassistant.components.weather import ATTR_CONDITION_CLEAR_NIGHT as ATTR_CONDITION_CLEAR_NIGHT, ATTR_CONDITION_CLOUDY as ATTR_CONDITION_CLOUDY, ATTR_CONDITION_EXCEPTIONAL as ATTR_CONDITION_EXCEPTIONAL, ATTR_CONDITION_FOG as ATTR_CONDITION_FOG, ATTR_CONDITION_HAIL as ATTR_CONDITION_HAIL, ATTR_CONDITION_LIGHTNING as ATTR_CONDITION_LIGHTNING, ATTR_CONDITION_LIGHTNING_RAINY as ATTR_CONDITION_LIGHTNING_RAINY, ATTR_CONDITION_PARTLYCLOUDY as ATTR_CONDITION_PARTLYCLOUDY, ATTR_CONDITION_POURING as ATTR_CONDITION_POURING, ATTR_CONDITION_RAINY as ATTR_CONDITION_RAINY, ATTR_CONDITION_SNOWY as ATTR_CONDITION_SNOWY, ATTR_CONDITION_SNOWY_RAINY as ATTR_CONDITION_SNOWY_RAINY, ATTR_CONDITION_SUNNY as ATTR_CONDITION_SUNNY, ATTR_CONDITION_WINDY as ATTR_CONDITION_WINDY
from homeassistant.const import CONCENTRATION_PARTS_PER_CUBIC_METER as CONCENTRATION_PARTS_PER_CUBIC_METER, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, LENGTH_FEET as LENGTH_FEET, LENGTH_INCHES as LENGTH_INCHES, LENGTH_METERS as LENGTH_METERS, LENGTH_MILLIMETERS as LENGTH_MILLIMETERS, PERCENTAGE as PERCENTAGE, SPEED_KILOMETERS_PER_HOUR as SPEED_KILOMETERS_PER_HOUR, SPEED_MILES_PER_HOUR as SPEED_MILES_PER_HOUR, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT, TIME_HOURS as TIME_HOURS, UV_INDEX as UV_INDEX
from typing import Final

API_IMPERIAL: Final[str]
API_METRIC: Final[str]
ATTRIBUTION: Final[str]
ATTR_FORECAST: Final[str]
CONF_FORECAST: Final[str]
DOMAIN: Final[str]
MANUFACTURER: Final[str]
MAX_FORECAST_DAYS: Final[int]
NAME: Final[str]
CONDITION_CLASSES: Final[dict[str, list[int]]]
FORECAST_SENSOR_TYPES: Final[tuple[AccuWeatherSensorDescription, ...]]
SENSOR_TYPES: Final[tuple[AccuWeatherSensorDescription, ...]]



from .const import BT_PREFIX as BT_PREFIX, CONF_REQUEST_RSSI as CONF_REQUEST_RSSI, DEFAULT_DEVICE_ID as DEFAULT_DEVICE_ID, DOMAIN as DOMAIN, SERVICE_UPDATE as SERVICE_UPDATE
from collections.abc import Awaitable
from homeassistant.components.device_tracker.const import CONF_SCAN_INTERVAL as CONF_SCAN_INTERVAL, CONF_TRACK_NEW as CONF_TRACK_NEW, DEFAULT_TRACK_NEW as DEFAULT_TRACK_NEW, SCAN_INTERVAL as SCAN_INTERVAL, SOURCE_TYPE_BLUETOOTH as SOURCE_TYPE_BLUETOOTH
from homeassistant.components.device_tracker.legacy import Device as Device, YAML_DEVICES as YAML_DEVICES, async_load_config as async_load_config
from homeassistant.const import CONF_DEVICE_ID as CONF_DEVICE_ID
from homeassistant.core import HomeAssistant as HomeAssistant, ServiceCall as ServiceCall
from homeassistant.helpers.event import async_track_time_interval as async_track_time_interval
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any, Callable, Final

_LOGGER: Final[Any]
PLATFORM_SCHEMA: Final[Any]

def is_bluetooth_device(device: Device) -> bool: ...
def discover_devices(device_id: int) -> list[tuple[str, str]]: ...
async def see_device(hass: HomeAssistant, async_see: Callable[..., Awaitable[None]], mac: str, device_name: str, rssi: Union[tuple[int], None] = ...) -> None: ...
async def get_tracking_devices(hass: HomeAssistant) -> tuple[set[str], set[str]]: ...
def lookup_name(mac: str) -> Union[str, None]: ...
async def async_setup_scanner(hass: HomeAssistant, config: ConfigType, async_see: Callable[..., Awaitable[None]], discovery_info: Union[dict[str, Any], None] = ...) -> bool: ...



from aiohttp.web import Request as Request, StreamResponse as StreamResponse
from aiohttp.web_urldispatcher import StaticResource
from collections.abc import Mapping as Mapping
from typing import Any, Final

CACHE_TIME: Final[Any]
CACHE_HEADERS: Final[Mapping[str, str]]

class CachingStaticResource(StaticResource):
    async def _handle(self, request: Request) -> StreamResponse: ...



from homeassistant.const import CONF_ELEVATION as CONF_ELEVATION, EVENT_CORE_CONFIG_UPDATE as EVENT_CORE_CONFIG_UPDATE, SUN_EVENT_SUNRISE as SUN_EVENT_SUNRISE, SUN_EVENT_SUNSET as SUN_EVENT_SUNSET
from homeassistant.core import callback as callback
from homeassistant.helpers import event as event
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.sun import get_astral_location as get_astral_location, get_location_astral_event_next as get_location_astral_event_next
from typing import Any

_LOGGER: Any
DOMAIN: str
ENTITY_ID: str
STATE_ABOVE_HORIZON: str
STATE_BELOW_HORIZON: str
STATE_ATTR_AZIMUTH: str
STATE_ATTR_ELEVATION: str
STATE_ATTR_RISING: str
STATE_ATTR_NEXT_DAWN: str
STATE_ATTR_NEXT_DUSK: str
STATE_ATTR_NEXT_MIDNIGHT: str
STATE_ATTR_NEXT_NOON: str
STATE_ATTR_NEXT_RISING: str
STATE_ATTR_NEXT_SETTING: str
PHASE_NIGHT: str
PHASE_ASTRONOMICAL_TWILIGHT: str
PHASE_NAUTICAL_TWILIGHT: str
PHASE_TWILIGHT: str
PHASE_SMALL_DAY: str
PHASE_DAY: str
_PHASE_UPDATES: Any

async def async_setup(hass, config): ...

class Sun(Entity):
    entity_id: Any
    hass: Any
    location: Any
    elevation: float
    _state: Any
    next_dawn: Any
    next_midnight: Any
    solar_elevation: Any
    rising: Any
    _next_change: Any
    def __init__(self, hass) -> None: ...
    @property
    def name(self): ...
    @property
    def state(self): ...
    @property
    def extra_state_attributes(self): ...
    phase: Any
    def _check_event(self, utc_point_in_time, sun_event, before): ...
    next_rising: Any
    next_noon: Any
    next_setting: Any
    next_dusk: Any
    def update_events(self, now: Any | None = ...) -> None: ...
    solar_azimuth: Any
    def update_sun_position(self, now: Any | None = ...) -> None: ...



from homeassistant.const import CONF_CLIENT_ID as CONF_CLIENT_ID, CONF_CLIENT_SECRET as CONF_CLIENT_SECRET, LENGTH_FEET as LENGTH_FEET, MASS_KILOGRAMS as MASS_KILOGRAMS, MASS_MILLIGRAMS as MASS_MILLIGRAMS, PERCENTAGE as PERCENTAGE, TIME_MILLISECONDS as TIME_MILLISECONDS, TIME_MINUTES as TIME_MINUTES
from typing import Final

ATTR_ACCESS_TOKEN: Final[str]
ATTR_REFRESH_TOKEN: Final[str]
ATTR_LAST_SAVED_AT: Final[str]
ATTR_DURATION: Final[str]
ATTR_DISTANCE: Final[str]
ATTR_ELEVATION: Final[str]
ATTR_HEIGHT: Final[str]
ATTR_WEIGHT: Final[str]
ATTR_BODY: Final[str]
ATTR_LIQUIDS: Final[str]
ATTR_BLOOD_GLUCOSE: Final[str]
ATTR_BATTERY: Final[str]
CONF_MONITORED_RESOURCES: Final[str]
CONF_CLOCK_FORMAT: Final[str]
ATTRIBUTION: Final[str]
FITBIT_AUTH_CALLBACK_PATH: Final[str]
FITBIT_AUTH_START: Final[str]
FITBIT_CONFIG_FILE: Final[str]
FITBIT_DEFAULT_RESOURCES: Final[list[str]]
DEFAULT_CONFIG: Final[dict[str, str]]
DEFAULT_CLOCK_FORMAT: Final[str]
FITBIT_RESOURCES_LIST: Final[dict[str, tuple[str, Union[str, None], str]]]
FITBIT_MEASUREMENTS: Final[dict[str, dict[str, str]]]
BATTERY_LEVELS: Final[dict[str, int]]



from .const import CONF_FORECAST as CONF_FORECAST, DOMAIN as DOMAIN
from homeassistant import config_entries as config_entries
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_API_KEY as CONF_API_KEY, CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE, CONF_NAME as CONF_NAME
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from typing import Any

class AccuWeatherFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    @staticmethod
    def async_get_options_flow(config_entry: ConfigEntry) -> AccuWeatherOptionsFlowHandler: ...

class AccuWeatherOptionsFlowHandler(config_entries.OptionsFlow):
    config_entry: Any
    def __init__(self, entry: ConfigEntry) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



from .const import DATA_CLIENT as DATA_CLIENT, DATA_LISTENER as DATA_LISTENER, DATA_PROTECTION_WINDOW as DATA_PROTECTION_WINDOW, DATA_UV as DATA_UV, DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_ATTRIBUTION as ATTR_ATTRIBUTION, CONF_API_KEY as CONF_API_KEY, CONF_BINARY_SENSORS as CONF_BINARY_SENSORS, CONF_ELEVATION as CONF_ELEVATION, CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE, CONF_SENSORS as CONF_SENSORS
from homeassistant.core import HomeAssistant as HomeAssistant, ServiceCall as ServiceCall, callback as callback
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import aiohttp_client as aiohttp_client
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect, async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.service import verify_domain_control as verify_domain_control
from pyopenuv import Client
from typing import Any

DEFAULT_ATTRIBUTION: str
NOTIFICATION_ID: str
NOTIFICATION_TITLE: str
TOPIC_UPDATE: Any
PLATFORMS: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...
async def async_migrate_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...

class OpenUV:
    client: Any
    data: Any
    def __init__(self, client: Client) -> None: ...
    async def async_update_protection_data(self) -> None: ...
    async def async_update_uv_index_data(self) -> None: ...
    async def async_update(self) -> None: ...

class OpenUvEntity(Entity):
    _attr_extra_state_attributes: Any
    _attr_should_poll: bool
    _attr_unique_id: Any
    _sensor_type: Any
    openuv: Any
    def __init__(self, openuv: OpenUV, sensor_type: str) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    def update_from_latest_data(self) -> None: ...



from .const import ATTR_FORECAST as ATTR_FORECAST, CONF_FORECAST as CONF_FORECAST, DOMAIN as DOMAIN
from aiohttp import ClientSession as ClientSession
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_API_KEY as CONF_API_KEY
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from typing import Any, Dict

_LOGGER: Any
PLATFORMS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def update_listener(hass: HomeAssistant, entry: ConfigEntry) -> None: ...

class AccuWeatherDataUpdateCoordinator(DataUpdateCoordinator[Dict[str, Any]]):
    location_key: Any
    forecast: Any
    is_metric: Any
    accuweather: Any
    def __init__(self, hass: HomeAssistant, session: ClientSession, api_key: str, location_key: str, forecast: bool) -> None: ...
    async def _async_update_data(self) -> dict[str, Any]: ...



from .discovery_data_template import BaseDiscoverySchemaDataTemplate as BaseDiscoverySchemaDataTemplate, DynamicCurrentTempClimateDataTemplate as DynamicCurrentTempClimateDataTemplate, ZwaveValueID as ZwaveValueID
from awesomeversion import AwesomeVersion
from collections.abc import Generator
from homeassistant.core import callback as callback
from typing import Any
from zwave_js_server.model.device_class import DeviceClassItem as DeviceClassItem
from zwave_js_server.model.node import Node as ZwaveNode
from zwave_js_server.model.value import Value as ZwaveValue

class DataclassMustHaveAtLeastOne:
    def __post_init__(self) -> None: ...

class FirmwareVersionRange(DataclassMustHaveAtLeastOne):
    min: Union[str, None]
    max: Union[str, None]
    min_ver: Union[AwesomeVersion, None]
    max_ver: Union[AwesomeVersion, None]
    def __post_init__(self) -> None: ...

class ZwaveDiscoveryInfo:
    node: ZwaveNode
    primary_value: ZwaveValue
    assumed_state: bool
    platform: str
    platform_hint: Union[str, None]
    platform_data_template: Union[BaseDiscoverySchemaDataTemplate, None]
    platform_data: Union[dict[str, Any], None]
    additional_value_ids_to_watch: Union[set[str], None]
    entity_registry_enabled_default: bool

class ZWaveValueDiscoverySchema(DataclassMustHaveAtLeastOne):
    command_class: Union[set[int], None]
    endpoint: Union[set[int], None]
    property: Union[set[Union[str, int]], None]
    property_name: Union[set[str], None]
    property_key: Union[set[Union[str, int]], None]
    property_key_name: Union[set[str], None]
    type: Union[set[str], None]

class ZWaveDiscoverySchema:
    platform: str
    primary_value: ZWaveValueDiscoverySchema
    hint: Union[str, None]
    data_template: Union[BaseDiscoverySchemaDataTemplate, None]
    manufacturer_id: Union[set[int], None]
    product_id: Union[set[int], None]
    product_type: Union[set[int], None]
    firmware_version_range: Union[FirmwareVersionRange, None]
    firmware_version: Union[set[str], None]
    device_class_basic: Union[set[Union[str, int]], None]
    device_class_generic: Union[set[Union[str, int]], None]
    device_class_specific: Union[set[Union[str, int]], None]
    required_values: Union[list[ZWaveValueDiscoverySchema], None]
    absent_values: Union[list[ZWaveValueDiscoverySchema], None]
    allow_multi: bool
    assumed_state: bool
    entity_registry_enabled_default: bool

def get_config_parameter_discovery_schema(property_: Union[set[Union[str, int]], None] = ..., property_name: Union[set[str], None] = ..., property_key: Union[set[Union[str, int]], None] = ..., property_key_name: Union[set[str], None] = ..., **kwargs: Any) -> ZWaveDiscoverySchema: ...

SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA: Any
SWITCH_BINARY_CURRENT_VALUE_SCHEMA: Any
SIREN_TONE_SCHEMA: Any
DISCOVERY_SCHEMAS: Any

def async_discover_values(node: ZwaveNode) -> Generator[ZwaveDiscoveryInfo, None, None]: ...
def check_value(value: ZwaveValue, schema: ZWaveValueDiscoverySchema) -> bool: ...
def check_device_class(device_class: DeviceClassItem, required_value: Union[set[Union[str, int]], None]) -> bool: ...
